/***************************************************************************
 *   Copyright (C) 2005 by Sean Harmer                                     *
 *   sh@rama.homelinux.org                                                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Steet, Fifth Floor, Boston, MA 02110-1301, USA.             *
 ***************************************************************************/
#ifndef KACLLISTVIEW_H
#define KACLLISTVIEW_H

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <kfileitem.h>
#include <klistview.h>
#include <qpixmap.h>

#include <sys/acl.h>

class KACL;
class KACLEntry;
class KACLListView;

class KACLListViewItem : public KListViewItem
{
    friend class KACLListView;
public:
    KACLListViewItem( QListView* parent, KACLEntry* pEntry = 0 );
    ~KACLListViewItem();

    void setACLEntry( acl_entry_t aclEntry );

    acl_tag_t entryType() const;

    void calcEffectiveRights();

    void togglePerm( acl_perm_t perm );

    virtual void paintCell( QPainter *p, const QColorGroup &cg,
                            int column, int width, int alignment );

    KACLEntry* entry() { return m_pEntry; };

    void updatePermPixmaps();
protected:
    KACLEntry* m_pEntry;
    KACLListView* m_pACLListView;
};

/**
@author Sean Harmer
*/
class KACLListView : public KListView
{
Q_OBJECT
public:
    enum ACLAvatar
    {
        AA_USER_OBJ = 0,
        AA_GROUP_OBJ,
        AA_OTHER,
        AA_USER,
        AA_GROUP,
        AA_MASK,
        AA_COUNT
    };

    KACLListView( KACL* pACL, QWidget* parent = 0, const char* name = 0 );
    ~KACLListView();

    bool hasMaskEntry() const { return m_bHasMask; }
    acl_perm_t maskPermissions() const;
    void setMaskPermissions( acl_perm_t maskPerms );
    acl_perm_t maskPartialPermissions() const;
    void setMaskPartialPermissions( acl_perm_t maskPerms );

    void calculateEffectiveRights();

    const KACL* acl() const { return m_pACL; }
    KACL* acl() { return m_pACL; }

    QPixmap m_aAvatars[AA_COUNT];
    QPixmap m_pmTick;
    QPixmap m_pmPartial;

public slots:
    void setACL( KACL* pACL );

protected slots:
    void entryClicked( QListViewItem* pItem, const QPoint& pt, int col );

private:
    KACL* m_pACL;
    KACLEntry* m_pMaskEntry;
    bool m_bHasMask;
};

#endif
