/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "applicationmenu.h"

#include <KDE/KIcon>
#include <KDE/KAction>
#include <KDE/KSycoca>
#include <KDE/KToolInvocation>

#include <kdebug.h>

ApplicationMenu::ApplicationMenu( QWidget *parent ) : KMenu(i18n("Applications"),parent)
{
    setObjectName("ApplicationMenu");
    connect(this, SIGNAL(triggered(QAction*)), this, SLOT(slotActionTriggered(QAction*)));
    connect( KSycoca::self(), SIGNAL(databaseChanged()), this, SLOT(slotRebuildMenus()));
    slotRebuildMenus();
}

int ApplicationMenu::populateMenu( KSharedPtr<KServiceGroup> group,  KMenu *parentMenu )
{
  int count = 0;
  if (!group || !group->isValid()) return count;

  group->setShowEmptyMenu(false);
  // Iterate over all entries in the group
  foreach( const KSharedPtr<KSycocaEntry> p, group->entries(true, true, false, true))
  {
    if (p->isType(KST_KService))
    {
        KSharedPtr<KService> a = KSharedPtr<KService>::staticCast(p);
        if( a->isApplication() )
        {

            QString appName;
            if( !a->genericName().isEmpty() )
                appName = i18n("%1 (%2)", a->genericName(), a->name());
            else
                appName = a->name();

            KAction *act = new KAction(KIcon( a->icon() ), appName, parentMenu);
            act->setData( p->entryPath() );
            parentMenu->addAction( act );
            count++;
        }
        else
            kDebug() << "Dunno here" << p->entryPath();
    }
    else if (p->isType(KST_KServiceGroup))
    {
      KSharedPtr<KServiceGroup> g = KSharedPtr<KServiceGroup>::staticCast(p);
      g->setShowEmptyMenu(false);
      if( g->entries(true,true).count() != 0 )
      {
          kDebug() << "Menu" << g->caption() << g->entries(true,true).count();
          KMenu *menu = new KMenu(parentMenu);
          menu->setIcon( KIcon( g->icon()  ));
          menu->setTitle( g->caption() );
          if( populateMenu( g, menu ) > 0 )
          {
              parentMenu->addMenu( menu );
              count++;
          }
          else
              delete menu;

      }
    }
    else
      kDebug() << "Dunno" << p->entryPath();

  }
  return count;
}

void ApplicationMenu::slotActionTriggered ( QAction * action )
{
    kDebug() << action->data().toString() << "clicked";
    QString errorString;
    if( KToolInvocation::startServiceByDesktopPath( action->data().toString(), QStringList(), &errorString ) > 0)
    {
        kDebug() << "Error:"  << errorString;
    }
}

 void ApplicationMenu::slotRebuildMenus( )
 {
     clear();
     populateMenu( KServiceGroup::root(), this);
 }

