/**************************************************************************
*   Copyright (C) 2009 by Ian Reinhart Geiser                             *
*   geiseri@kde.org                                                       *
*                                                                         *
*   This program is free software; you can redistribute it and/or modify  *
*   it under the terms of the GNU General Public License as published by  *
*   the Free Software Foundation; either version 2 of the License, or     *
*   (at your option) any later version.                                   *
*                                                                         *
*   This program is distributed in the hope that it will be useful,       *
*   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
*   GNU General Public License for more details.                          *
*                                                                         *
*   You should have received a copy of the GNU General Public License     *
*   along with this program; if not, write to the                         *
*   Free Software Foundation, Inc.,                                       *
*   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
***************************************************************************/

#include "clock.h"
#include <KDE/KLocale>
#include <KDE/KApplication>
#include <KDE/KGlobalSettings>
#include <KDE/KPopupFrame>
#include <KDE/KDatePicker>
#include <KDE/KCMultiDialog>


#include <QTimer>
#include <QDateTime>
#include <QMouseEvent>
#include <QPainter>

Clock::Clock(QWidget *parent ) : QFrame(parent)
{
    setObjectName("Clock");

    m_timer = new QTimer(this);
    connect(m_timer, SIGNAL(timeout()), SLOT(updateTime()));
    m_timer->start(5000);  // signal every 5 seconds
    updateTime();
}

void Clock::paintEvent(QPaintEvent *ev)
{
    QFont clockFont = KGlobalSettings::smallestReadableFont();

    QPainter painter(this);
    painter.setWindow(0, 0, width(), height());
    // if its available use it
    painter.setRenderHints(QPainter::Antialiasing|QPainter::TextAntialiasing);
    painter.setPen( palette().color( QPalette::WindowText )); //clock color
    painter.setFont( clockFont );
    painter.drawText(0, 0, width(), height() / 2, Qt::AlignCenter|Qt::AlignVCenter, m_stringTime); //clock
    painter.drawText(0, height() / 2 , width(), height() / 2 , Qt::AlignCenter|Qt::AlignVCenter, m_stringDate); //date
}

void Clock::updateTime()
{
    QFont clockFont = KGlobalSettings::smallestReadableFont();
    QFontMetrics metrics( clockFont );
    m_stringTime = KGlobal::locale()->formatTime(QTime::currentTime());
    m_stringDate = KGlobal::locale()->formatDate(QDate::currentDate());
    int timeW = metrics.width(m_stringTime);
    int timeH = metrics.height();
    int dateW = metrics.width(m_stringDate);
    int dateH = metrics.height();

    setFixedWidth(  qMax( timeW, dateW ) + 4 );
    update();
}

void Clock::mousePressEvent(QMouseEvent *ev)
{
    if( ev->button() == Qt::LeftButton )
    {
        KPopupFrame calenderPopup(this);
        KDatePicker *calender = new KDatePicker(&calenderPopup);
        calenderPopup.setMainWidget(calender);
	calender->adjustSize();
        calenderPopup.adjustSize();
        calenderPopup.exec( mapToGlobal( ev->pos() ) );
    }
    else if( ev->button() == Qt::RightButton )
    {
        KCMultiDialog timeConf(this);
        timeConf.addModule(QLatin1String("clock"));
        timeConf.exec();
    }
}
