/*
    Copyright (C) 2006,2007 Olivier Goffart <ogoffart at kde.org>

    This program is free software; you can redistribute it and/or
    modify it under the terms of the GNU General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.

*/

#include <QLabel>
#include <QLayout>
#include <QVBoxLayout>
#include <QFrame>
#include <QHBoxLayout>


#include <kapplication.h>
#include <kcombobox.h>
#include <kconfig.h>
#include "kcolorsconfigwidget.h"
#include <kparts/genericfactory.h>
#include <kstandarddirs.h>
#include <kurlcompletion.h>
#include <kurlrequester.h>

#include "kcmcolors.h"


typedef KGenericFactory<KCMColors, QWidget> ColorsFactory;
K_EXPORT_COMPONENT_FACTORY( colors, ColorsFactory("kcmcolors") )
		

KCMColors::KCMColors(QWidget *parent, const QStringList & )
    : KCModule(ColorsFactory::componentData(), parent/*, name*/)
{
    setButtons( Help | Default | Apply );
	
	 setQuickHelp( i18n("<h1>Colors</h1> This module allows you to choose"
       " the color scheme used for the KDE desktop. The different"
       " elements of the desktop, such as title bars, menu text, etc.,"
       " are called \"widgets\". You can choose the widget whose"
       " color you want to change by selecting it from a list, or by"
       " clicking on a graphical representation of the desktop.<p>"
       " You can save color settings as complete color schemes,"
       " which can also be modified or deleted. KDE comes with several"
       " predefined color schemes on which you can base your own.<p>"
       " All KDE applications will obey the selected color scheme."
       " Non-KDE applications may also obey some or all of the color"
       " settings, if this option is enabled."));

	QVBoxLayout *app_layout = new QVBoxLayout( this );

	QLabel *label = new QLabel( i18n( "Application :" ), this );
	m_appCombo = new KComboBox( false, this );
	QHBoxLayout *hbox = new QHBoxLayout();
	app_layout->addItem( hbox );
	hbox->addWidget( label );
	hbox->addWidget( m_appCombo, 10 );

	m_colorConfigWidget = new KColorsConfigWidget( this );
	app_layout->addWidget( m_colorConfigWidget );

    connect( m_colorConfigWidget, SIGNAL( changed( bool )), SIGNAL( changed(bool)));
	changed (true);

	connect( m_appCombo, SIGNAL( activated( const QString& ) ),
			SLOT( slotAppActivated( const QString& )) );

	KAboutData* ab = new KAboutData(
        "kcmcolors", 0, ki18n("KDE Colors Control Module"), "2.0",
        ki18n("System Notification Control Panel Module"),
        KAboutData::License_GPL, ki18n("(c) 2006-2007 KDE Team"));

	ab->addAuthor( ki18n("Olivier Goffart"), KLocalizedString(), "ogoffart@kde.org" );
	setAboutData( ab );

	load();


}

KCMColors::~KCMColors()
{
	/*KConfig config( "kcmcolorsrc", false, false );
	config.setGroup( "Misc" );
	config.writeEntry( "LastConfiguredApp", m_appCombo->currentText() );
	config.sync();*/
}

void KCMColors::slotAppActivated( const QString& text )
{
	m_colorConfigWidget->save();
	m_colorConfigWidget->setApplication( text );
	emit changed(true);
}



void KCMColors::defaults()
{
//    m_colorConfigWidget->resetDefaults( true ); // ask user
}

void KCMColors::load()
{
	m_appCombo->clear();
	QStringList fullpaths =
		KGlobal::dirs()->findAllResources("data", "*/*.kcolorsrc", KStandardDirs::NoDuplicates );
	foreach (const QString &fullPath, fullpaths )
	{
		int slash = fullPath.lastIndexOf( '/' ) - 1;
		int slash2 = fullPath.lastIndexOf( '/', slash );
		QString appname= slash2 < 0 ? QString() :  fullPath.mid( slash2+1 , slash-slash2  );
		if ( !appname.isEmpty() )
			m_appCombo->addItem( appname );
	}
	
	/*KConfig config( "kcolorsrc", true, false );
	config.setGroup( "Misc" );
	QString appDesc = config.readEntry( "LastConfiguredApp", "KDE System Notifications" );

	if this code gets enabled again, make sure to apply r494965
	*/


	slotAppActivated( m_appCombo->currentText() );

    // unsetCursor(); // unsetting doesn't work. sigh.
	setEnabled( true );
	emit changed( false );

	emit changed(true);

}

void KCMColors::save()
{

    m_colorConfigWidget->save();

    emit changed( true );
}


#include "kcmcolors.moc"
