/*
 * KDE Composite Manager, 
 * Copyright (C) 2003-2005 Fredrik Höglund <fredrik@kde.org>
 *
 * Based on xcompmgr,
 * Copyright (C) 2003 Keith Packard <keithp@keithp.com>
 *
 * Internal structure and naming style inspired by KWin,
 * Copyright (C) 1999, 2000 Matthias Ettrich <ettrich@kde.org>
 * Copyright (C) 2003 Lubos Lunak <l.lunak@kde.org>
 *
 * Permission to use, copy, modify, distribute, and sell this software and its
 * documentation for any purpose is hereby granted without fee, provided that
 * the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of Fredrik Höglund not be used in
 * advertising or publicity pertaining to distribution of the software without
 * specific, written prior permission.  Fredrik Höglund makes no
 * representations about the suitability of this software for any purpose.  It
 * is provided "as is" without express or implied warranty.
 *
 * FREDRIK HÖGLUND DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE,
 * INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO
 * EVENT SHALL FREDRIK HÖGLUND BE LIABLE FOR ANY SPECIAL, INDIRECT OR
 * CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM LOSS OF USE,
 * DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR OTHER
 * TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
 * PERFORMANCE OF THIS SOFTWARE.
 */

#include <kcmdlineargs.h>
#include <klocale.h>
#include <kdebug.h>
#include <dcopclient.h>

#include "main.h"
#include "workspace.h"
#include "extensions.h"

#include <X11/Xlib.h>

Atom net_current_desktop;
Atom net_active_window;

Atom net_wm_window_opacity;
Atom net_wm_desktop;

Atom net_wm_window_type;
Atom net_wm_window_type_desktop;
Atom net_wm_window_type_dock;
Atom net_wm_window_type_toolbar;
Atom net_wm_window_type_menu;
Atom net_wm_window_type_utility;
Atom net_wm_window_type_splash;
Atom net_wm_window_type_dialog;
Atom net_wm_window_type_normal; 
Atom net_wm_window_type_override; // KDE extension for no-border windows
Atom net_wm_window_type_topmenu;  // KDE extension for mac style menubar

Atom xa_xrootpmap_id;
Atom xa_xsetroot_id;
Atom xa_esetroot_pmap_id;

Display *dpy;


static int x11ErrorHandler( Display *, XErrorEvent * )
{
	// Do nothing for now
	return 0;
}

Application::Application()
		: KApplication()
{
	dpy = QX11Info::display();

	// Install our own X11 error handler
	XSetErrorHandler( x11ErrorHandler );

	const char *names[18];
	Atom atoms_return[18];
	Atom *atoms[18];
	int n = 0;

	names[n]   = "_NET_WM_WINDOW_TYPE";
	atoms[n++] = &net_wm_window_type;

	names[n]   = "_NET_WM_WINDOW_TYPE_DESKTOP";
	atoms[n++] = &net_wm_window_type_desktop;

	names[n]   = "_NET_WM_WINDOW_TYPE_DOCK";
	atoms[n++] = &net_wm_window_type_dock;

	names[n]   = "_NET_WM_WINDOW_TYPE_TOOLBAR";
	atoms[n++] = &net_wm_window_type_toolbar;

	names[n]   = "_NET_WM_WINDOW_TYPE_MENU";
	atoms[n++] = &net_wm_window_type_menu;

	names[n]   = "_NET_WM_WINDOW_TYPE_UTILITY";
	atoms[n++] = &net_wm_window_type_utility;

	names[n]   = "_NET_WM_WINDOW_TYPE_SPLASH";
	atoms[n++] = &net_wm_window_type_splash;

	names[n]   = "_NET_WM_WINDOW_TYPE_DIALOG";
	atoms[n++] = &net_wm_window_type_dialog;

	names[n]   = "_NET_WM_WINDOW_TYPE_NORMAL";
	atoms[n++] = &net_wm_window_type_normal; 

	names[n]   = "_KDE_NET_WM_WINDOW_TYPE_OVERRIDE";
	atoms[n++] = &net_wm_window_type_override;

	names[n]   = "_KDE_NET_WM_WINDOW_TYPE_TOPMENU";
	atoms[n++] = &net_wm_window_type_topmenu;

	names[n]   = "_NET_WM_WINDOW_OPACITY";
	atoms[n++] = &net_wm_window_opacity;

	names[n]   = "_NET_WM_DESKTOP";
	atoms[n++] = &net_wm_desktop;

	names[n]   = "_NET_CURRENT_DESKTOP";
	atoms[n++] = &net_current_desktop;

	names[n]   = "_NET_ACTIVE_WINDOW";
	atoms[n++] = &net_active_window;

	names[n]   = "_XROOTPMAP_ID";
	atoms[n++] = &xa_xrootpmap_id;

	names[n]   = "ESETROOT_PMAP_ID";
	atoms[n++] = &xa_esetroot_pmap_id;

	names[n]   = "_XSETROOT_ID";
	atoms[n++] = &xa_xsetroot_id;

	XInternAtoms( dpy, (char**)names, n, false, atoms_return );
	for ( int i = 0; i < n; i++ )
		*atoms[i] = atoms_return[i];
}


Application::~Application()
{
}


bool Application::x11EventFilter( XEvent *event )
{
	if ( Workspace::instance()->x11Event( event ) )
		return true;

	return KApplication::x11EventFilter( event );
}


int main( int argc, char **argv )
{
	KCmdLineArgs::init( argc, argv, "kcompmgr", 0, ki18n("kcompmgr"), "0.1" , ki18n("KDE Composite Manager"));

	KApplication::disableAutoDcopRegistration();
	Application app;

	if ( !Extensions::initialize() )
		return 1;

	app.dcopClient()->registerAs( "kcompmgr", false );
	app.dcopClient()->setDefaultObject( "CMInterface" );
	app.exec();
}

