/***************************************************************************
 *   Copyright (C) 2008-2009 by Marcel Hasler <mahasler_at_gmail.com>      *
 *   Copyright (C) 2010 by Ozan Çağlayan <ozan_at_pardus.org.tr>           *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#ifndef OSD_H
#define OSD_H

#include <QTimer>
#include <QDesktopWidget>

#include <KDE/Plasma/Label>
#include <KDE/Plasma/Meter>
#include <KDE/Plasma/Dialog>
#include <KDE/Plasma/FrameSvg>
#include <KDE/Plasma/IconWidget>

class OSD : public Plasma::Dialog
{
    Q_OBJECT
public:
        OSD();
        ~OSD();

        void setSize(unsigned int percent);
        void setOpacity(unsigned int percent);
        void setTimeout(unsigned int seconds);
        void setPrimaryScreenLock(bool lock);
        void setPosition(unsigned int xPercent, unsigned int yPercent);

        void display(QString icon, QString text = "", unsigned int percent = 0);

private slots:
        void slotHide();
        void slotReloadTheme();

private:
        void updatePosition();
        void initLayout();

        Plasma::FrameSvg m_backPanel;
        QRect m_rect;

        Plasma::Label *m_labelText;
        Plasma::IconWidget *m_labelIcon;

        QRect m_iconRect;
        QRect m_labelRect;

        QFont m_labelFont;
        QColor m_labelColor;

        Plasma::Meter *m_meter;

        QTimer m_timer;

        unsigned int m_hPos, m_vPos; // Relative positions (in percent)
        unsigned int m_xPos, m_yPos; // Absolute positions
        unsigned int m_width, m_height;
        unsigned int m_timeout;

        float m_scaleFactor;
        float m_opacity;

        bool m_isVisible;
        bool m_lockPrimaryScreen;
};

#endif // OSD_H
