/*
    This file is part of KDE.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
    
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
    GNU General Public License for more details.
    
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

#include "appview.h"

#include "entry.h"
#include "appmanager.h"

#include <klocale.h>
#include <kdialog.h>

#include <qlayout.h>
#include <qpushbutton.h>
#include <qtextview.h>
#include <qlabel.h>

AppView::AppView( QWidget *parent, AppManager *a )
  : QWidget( parent ), mAppManager( a ), mCurrentEntry( 0 )
{
  QBoxLayout *topLayout = new QVBoxLayout( this );
  topLayout->setSpacing( KDialog::spacingHint() );
  topLayout->setMargin( KDialog::marginHint() );
  
  mTextView = new QTextView( this );
  topLayout->addWidget( mTextView );
  
  QBoxLayout *buttonLayout = new QHBoxLayout( topLayout );

  mStatusLabel = new QLabel( this );
  buttonLayout->addWidget( mStatusLabel );
  
  buttonLayout->addStretch( 1 );
  
  QPushButton *button = new QPushButton( i18n("Install"), this );
  buttonLayout->addWidget( button );
  connect( button, SIGNAL( clicked() ), SLOT( install() ) );
}

void AppView::showEntry( KNS::Entry *e )
{
  setStatusMessage( QString::null );

  mCurrentEntry = e;

  QString txt;
  txt += "<h1>" + e->name() + "</h1>";
  txt += "<p>" + e->summary() + "</p>";
  txt += "<p><b>Author:</b> " + e->author() + "</p>";
  txt += "<p><b>Version:</b> " + e->version() + "</p>";

  txt += "<br/>";
  
  QString installedVersion = mAppManager->installedVersion( e->name() );
  
  if ( installedVersion.isEmpty() ) {
    txt += "<em>New Application</em>";
  } else {
    txt += "<p><b>Installed Version:</b> " + installedVersion + "</p>";
    if ( mAppManager->hasNewerVersion( e->name(), installedVersion ) ) {
      txt += "<p><font color=\"red\">Update available</font></p>";
    }
  }
  
  mTextView->setText( txt );
}

void AppView::install()
{
  setStatusMessage( i18n("Installing...") );
  emit install( mCurrentEntry );
}

void AppView::setStatusMessage( const QString &txt )
{
  mStatusLabel->setText( txt );
}

#include "appview.moc"
