/* This file is part of the KDE project
   Copyright (C) 2005 Ramakrishna R <coolrama@gmail.com>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
   Boston, MA 02111-1307, USA.
*/
#include"lbapi.h"
#include<iostream>
#include<mysql++/mysql++.h>

using namespace mysqlpp;
using namespace std;
using namespace KLB;

const char * labelListName = "KLabelBrowserTable";
const char * userName = "klb";
const char * tempPassword = "password";
const char * hostName = "localhost";

LabelManager::LabelManager()
{
	Connection con = new Connection();
	try
	{
		con.connect("mysql",hostName,userName,tempPassword);
	       			//assuming "mysql" table is present by default
		con.select_db(labelListName);
		//assuming the database *labelListName already exists. if it does not exist then 
		//create the database and all the tables inside the catch block
	}//try
	catch (mysqlpp::BadQuery &) 
	{
		if (con.create_db(labelListName)) 
		{
			cerr << "Failed to create Label List " <<con.error() << endl;
		}
		else
		{
			con.select_db(labelListName);
			Query query = con.query();

			query.execute("create table LabelNames (LabelId mediumint unsigned not null AUTO_INCREMENT primary key, names varchar(255) unique, Hidden integer)");

			query.execute("create table List (LabelId mediumint unsigned not null , Path varchar(255),primary key(LabelId,Path)) ");
			
			query.execute("create table Comment (LabelId mediumint unsigned not null , Comment varchar(255)) ");
			query.execute("create table Keywords (KeywordId mediumint unsigned not null AUTO_INCREMENT primary key , Keyword varchar(255) unique) ");
			query.execute("create table KeywordLabelMap (LabelId mediumint unsigned not null , KeywordId mediumint unsigned not null, primary key(LabelId,KeywordId)) ");
			
		}
		/*
		else if (!con.connect(labelListName,hostName,userName,tempPassword)) 
		{
			cerr << "Failed to access Label List" << endl;	
		}*/
	} 
	catch (exception& er)
	{
		cerr << er.what() << endl;
	}
}

/*
bool LabelManager::createLabel(const char *labelName)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
 	try
    	{
		query << "insert into LabelNames(names) values (\""<<labelName<<"\")";
		query.execute();
    	}
	catch(exception& er)
    	{
		cerr << er.what() << endl;
	}
	return 0;	
}//createNewLabel
*/


bool LabelManager::createLabel(const QString &labelName,const QString &comment,hiddenStatus hidden)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	mysqlpp::Result result;
 	try
    	{
		query << "insert into LabelNames(names,Hidden) values (\""<<labelName<<"\","<<hidden<<")";
		query.execute();
		query.reset();
		query << "select LabelId from LabelNames where names = \""<<labelName<<"\"";
		result = query.store();
		//cout << result.at(0).at(0);
		query.reset();
		query<< "insert into Comment(LabelId,Comment) values ("<<result.at(0).at(0)<<",\""<<comment<<"\")";
		query.execute();

    	}
	catch(exception& er)
    	{
		cerr << er.what() << endl;
	}
	return 0;	
}


bool LabelManager::createLabel(const QString &labelName,hiddenStatus hidden)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
 	try
    	{
		query << "insert into LabelNames(names,Hidden) values (\""<<labelName<<"\","<<hidden<<")";
		query.execute();
    	}
	catch(exception& er)
    	{
		cerr << er.what() << endl;
	}
	return 0;	
}

bool LabelManager::applyLabel(const QString & labelName,const QString & path)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	mysqlpp::Result result;
	try
	{
		query << "select LabelId from LabelNames where names = \""<<labelName<<"\"";
		result = query.store();
		//cout << result.at(0).at(0);
		query.reset();
		query<< "insert into List(LabelId,Path) values ("<<result.at(0).at(0)<<",\""<<path<<"\")";
		query.execute();

	}
	catch(exception& er)
	{
		cerr << er.what() << endl;
	}
	return 0;
}

bool LabelManager::getAllLabels(QStringList &qlabelList,KLB::hiddenStatus hidden)
{
	QString labelList;
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	stringstream resultstr;
	mysqlpp::Result result;
	try
	{
		if(hidden==KLB::NOT_HIDDEN)
		{
			query << "select names from LabelNames where Hidden="<<NOT_HIDDEN;	
		}
		else
		{
			query << "select names from LabelNames";
		}
		result = query.store();
//		labelList = "";
		int i;
		for (i=0;i<result.size()-1;i++)
		{
			resultstr << result.at(i).at(0) << ",";
		}
		if(0!=result.size())
			resultstr<< result.at(i).at(0); // last name without coma.

		labelList = resultstr.str();
		// conversion from QStrint to QStringList	
		QStringList list ;
		list = list.split(",", labelList);
		QString temp;
	
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	       {
		    temp = *it;
		    temp = temp.mid(1,temp.length()-2);
		    qlabelList << temp; 
	       }
	}
	catch(exception& er)
	{
		cerr << er.what() << endl;
		return 1;
	}
	return 0;
}

bool LabelManager::getLabels(const QString & path, QStringList & qlabelList)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	QString temp;
	stringstream resultstr;
	QString labelList;
	mysqlpp::Result result;
	try
	{
		query << "select names from LabelNames, List where List.LabelId=LabelNames.LabelId and List.Path= \""<<path<<"\"";
		result = query.store();
//		cout << result.at(0).at(0) << result.size() ;
		labelList = "";
		for (int i=0;i<result.size()-1;i++)
		{
			resultstr << result.at(i).at(0) <<",";
		//	temp = result.at(i).at(0); //first column
		//	labelList = labelList + temp;
		}
	       labelList = resultstr.str();
	
    		QStringList list ;
		list = list.split(",", labelList);
		QString temp;
	
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	        {
		    temp = *it;
		    temp = temp.mid(1,temp.length()-2);
		    qlabelList << temp; 
		}
	}
	catch(exception& er)
	{
		cerr << er.what() << endl;
		return 1;
	}
	return 0;
}

bool LabelManager::getPaths(const QString &label, QStringList & pathList)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	stringstream resultstr;
	QString path;
	mysqlpp::Result result;
	try
	{
		query << "select Path from LabelNames, List where List.LabelId=LabelNames.LabelId and names= \""<<label<<"\"";
		//cout << query.preview();
		result = query.store();
		for (int i=0;i<result.size();i++)
		{
			resultstr << result.at(i).at(0)<<",";
		}
		path = resultstr.str();
         	
                QStringList list ;
		list = list.split(",", path);
		QString temp;
	
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	        {
		    temp = *it;
		    temp = temp.mid(1,temp.length()-2);
		    pathList << temp; 
		}

	}
	catch(exception& er)
	{
		cerr << er.what() << endl;
		return 1;
	}
	return 0;
}

bool LabelManager::addKeywords(const QString &labelName,const QStringList & keywordList)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	mysqlpp::Result result;
	mysqlpp::Result kresult;
	try
	{
		query << "select LabelId from LabelNames where names = \""<<labelName<<"\"";
		result = query.store();
		//cout << result.at(0).at(0);
	  	QStringList kl = (QStringList)keywordList;	
		
/*
		query.reset();
		query << "select KeywordId from Keywords where Keyword = \""<<labelName<<"\"";
		kresult = query.store();
		if(kresult.size()==0)
		{
			query.reset();
			query<< "insert into Keywords(Keyword) values ("<<*it<<")";
			query.execute();
		}
*/		
		
		
/***********

		//cout << query.preview();
		result = query.store();
		for (int i=0;i<result.size();i++)
		{
			resultstr << result.at(i).at(0)<<",";
		}
		path = resultstr.str();

*************/

		for ( QStringList::Iterator it = kl.begin(); it != kl.end(); ++it )
		{
			query.reset();
			query << "select KeywordId from Keywords where Keyword = \""<<*it<<"\"";
			kresult = query.store();
			if(kresult.size()==0)
			{
				query.reset();
				query<< "insert into Keywords(Keyword) values (\""<<*it<<"\")";
				query.execute();
			}
	
			query.reset();
			query << "select KeywordId from Keywords where Keyword = \""<<*it<<"\"";
			kresult = query.store();
			query.reset();
			query <<" select LabelId from LabelNames where names = \""<<labelName<<"\" " ;
			result = query.store();
			query.reset();
			query<< "insert into KeywordLabelMap(LabelId,KeywordId) values ("<<result.at(0).at(0)<<","<<kresult.at(0).at(0)<<")";
			query.execute();
		}
	}
	catch(exception& er)
	{
		cerr << "caught an exception"<< er.what() << endl;
	}
	return 0;

}

bool LabelManager::searchLabels(QStringList & pathList , const QString &searchQuery)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	QString temppath;
	stringstream resultstr;
	mysqlpp::Result result;
	try
	{
	
		//QStringList temppresentLabels = (QStringList)presentLabels;
		query<< "select distinct Path from LabelNames, List where List.LabelId=LabelNames.LabelId and ("  << searchQuery << ")";

/*
		for(QStringList::Iterator i2 = temppresentLabels.begin(); i2!=temppresentLabels.end();++i2)
		{
			if(i2==temppresentLabels.begin())
			{
				query << "names = \""<<*i2<<"\"";
			}
			else
			{	
				cout<< " *it is "<<*i2<<endl;
				query << "or names= \""<<*i2<<"\"";
			}
		}
		query << ")";
*/
		cout << query.preview() << endl;
		result = query.store();
		for (int i=0;i<result.size();i++)
		{
			resultstr << result.at(i).at(0)<<",";
		}
		temppath = resultstr.str();
         	
                QStringList list ;
		list = list.split(",", temppath);
		QString temp;
	
		for ( QStringList::Iterator it = list.begin(); it != list.end(); ++it )
	        {
		    temp = *it;
		    temp = temp.mid(1,temp.length()-2);
		    pathList << temp; 
		}

	}
	catch(exception& er)
	{
		cerr << er.what() << endl;
		return 1;
	}
	return 0;

}

bool  LabelManager::scanKeywordsAndAutomaticApply(const QString & path , const QStringList &words)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	mysqlpp::Result result;
	mysqlpp::Result tempresult;
	try
	{
		
		QStringList * tempwords = &(QStringList)words;
		for ( QStringList::Iterator it = tempwords->begin(); it != tempwords->end(); ++it )
		{
			query.reset();
			query << "select LabelNames.LabelId from  LabelNames, KeywordLabelMap, Keywords where KeywordLabelMap.LabelId=LabelNames.LabelId and Keywords.KeywordId=KeywordLabelMap.KeywordId and Keywords.Keyword = \""<<*it<<"\"";
			result = query.store();
			int i;
			for(i = 0; i < result.size() ;i++)
			{
				query.reset();
				query << "select * from List where LabelId="<<result.at(i).at(0)<<" and Path=\""<<path<<"\"";
				tempresult = query.store();
				if(tempresult.size()==0)
				{
					query.reset();
					query<< "insert into List(LabelId,Path) values ("<<result.at(i).at(0)<<",\""<<path <<"\")";
					query.execute();
				}
			} 
		}
	}
	catch(exception& er){
		cerr << "caught an exception"<< er.what() << endl;
	}
	return 0;
}

bool  LabelManager::deleteLabel(const QString &labelName)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	Result tempresult;
	
	query.reset();
	query << "select LabelId from LabelNames where names=\""<<labelName<<"\"";
	tempresult = query.store();
	if(tempresult.size()==0)
	{
		return false;
	}
	else
	{
		query.reset();
		query << "delete from KeywordLabelMap where LabelId="<<tempresult.at(0).at(0);
		query.execute();
		query.reset();
		query << "delete from List where LabelId="<<tempresult.at(0).at(0);
		query.execute();
		query.reset();
		query << "delete from LabelNames where LabelId="<<tempresult.at(0).at(0);
		query.execute();
		query.reset();
		query << "delete from Comment where LabelId="<<tempresult.at(0).at(0);
		query.execute();
	}
	return true;
}

bool  LabelManager::renameLabel(const QString &oldLabelName,const QString &newLabelName)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	Result tempresult;
	
	query.reset();
	query << "select LabelId from LabelNames where names=\""<<oldLabelName<<"\"";
	tempresult = query.store();
	if(tempresult.size()==0)
	{
		return false;
	}
	else
	{
		query.reset();
		query << "update LabelNames set names=\""<<newLabelName<<"\" where names=\""<<oldLabelName<<"\"";
		query.execute();
	}
	return true;
}

bool  LabelManager::removeLabel(const QString &LabelName,const QString &path)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	Result tempresult;

    try{
	
	query.reset();
	query << "select List.LabelId from List,LabelNames where List.LabelId=LabelNames.LabelId and List.Path=\""<<path<<"\" and LabelNames.names =\""<<LabelName<<"\"";
	tempresult = query.store();
	cout<<" select query executed"<<endl;
	
	if(tempresult.size()==0)
	{
		return false;
	}
	else
	{
		query.reset();
		cout<<"before delete query executed"<<endl;
		query << "delete from List where LabelId="<<tempresult.at(0).at(0)<<" and Path=\""<<path<<"\"";
		query.execute();
	}
     }
     catch(exception &er)
     {
	cout<<"caught exception" << er.what() << endl;
     }
	return true;
}

bool  LabelManager::getAllXML(QString &xmlData)
{
	Connection con = new Connection();
	con.connect(labelListName,hostName,userName,tempPassword);
	mysqlpp::Query query = con.query();
	mysqlpp::Result result;
	mysqlpp::Result result2;
	stringstream resultstr;
	stringstream resultstr2;
	xmlData ="";
	xmlData +="<KLB>";

	xmlData +="<AllLabels>";

	try{
		query.reset();
		query << "select LabelId,names from LabelNames";
		result = query.store();
		for (int i=0;i<result.size();i++)
		{	
			resultstr << "<Label>";
			resultstr <<"<LabelId>"<< result.at(i).at(0)<<"</LabelId>";
			resultstr <<"<name>" << result.at(i).at(1) << "</name>" ; 
			resultstr << "</Label>";
		}
		QString temp;
		temp =  resultstr.str();
		
		xmlData += temp;	
	}catch(exception &er)
     	{
		cout<<"caught exception" << er.what() << endl;
		
	}

	xmlData +="</AllLabels>";

	xmlData +="<AllPaths>";

	try{
		query.reset();
		query << "select distinct Path from List";
		result = query.store();
		int j;

		for (int i=0;i<result.size();i++)
		{	
			resultstr2 << "<pathLabels>";
			query.reset();
			resultstr2<<"<path>";
			resultstr2<<result.at(i).at(0);	
			resultstr2<<"</path>";
	
			query << "select LabelId from List where Path="<<result.at(i).at(0);
			 cout <<"\n\n" <<query.preview() << endl;
			result2  = query.store();
			for(j=0 ; j < result2.size();j++)
			{
				resultstr2 <<"<LabelId>"<< result2.at(j).at(0)<<"</LabelId>";
			}
			resultstr2 << "</pathLabels>";
		}
		QString temp;
		temp =  resultstr2.str();
		
		xmlData += temp;	
	}catch(exception &er)
     	{
		cout<<"caught exception" << er.what() << endl;
		
	}

	xmlData +="</AllPaths>";



	xmlData +="</KLB>";
	 	
    	return true;
}

