/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/


#include "mergerwidget.h"
#include "mergermodel.h"
#include "resolutiondialog.h"
#include "mergerinterface.h"

Nepomuk::MergerWidget::MergerWidget(QWidget* parent)
    : QWidget( parent )
{
    setupUi( this );

    m_model = new MergerModel( this );
    m_view->setModel( m_model );

    // DBus
    m_merger = new Merger( QLatin1String("org.kde.nepomuk.services.nepomukbackupsync"),
                           QLatin1String("/merger"),
                           QDBusConnection::sessionBus() );
    if( m_merger->isValid() ) {
        connect( m_merger, SIGNAL(multipleMerge(QString,QString)),
                 m_model, SLOT(multipleMerge(QString,QString)) );
    }

    connect( m_ignoreAll, SIGNAL(clicked(bool)), this, SLOT(ignoreAllClicked()) );
    connect( m_ignore, SIGNAL(clicked(bool)), this, SLOT(ignoreClicked()) );
    
    connect( m_resolve, SIGNAL(clicked(bool)), this, SLOT(resolveClicked()) );
    connect( m_view, SIGNAL(activated(QModelIndex)), this, SLOT(resolveClicked()) );
}

Nepomuk::MergerWidget::~MergerWidget()
{
    delete m_model;
}

void Nepomuk::MergerWidget::ignoreAllClicked()
{
    m_model->clearAll();
}

void Nepomuk::MergerWidget::ignoreClicked()
{
    m_model->remove( m_view->currentIndex() );
}

void Nepomuk::MergerWidget::resolveClicked()
{
    MergerRecord mr = m_model->record( m_view->currentIndex() );
    ResolutionDialog * dialog = new ResolutionDialog( mr.uri, mr.prop, this );
    dialog->show();
    dialog->exec();
    dialog->deleteLater();
}


