/*
    This file is part of the Nepomuk KDE project.
    Copyright (C) 2010  Vishesh Handa <handa.vish@gmail.com>

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#ifndef SYNCREQUEST_H
#define SYNCREQUEST_H

#include <QtCore/QPair>
#include <QtCore/QUrl>
#include <QtCore/QMultiHash>
#include <QtCore/QDateTime>
#include <QtCore/QQueue>

#include <Nepomuk/ResourceManager>


namespace Soprano {
    class Statement;
    class Model;
}

namespace Nepomuk {
    namespace Sync {
        class ChangeLog;
        class ChangeLogRecord;
    }
    
    class SyncRequest : public QObject
    {
        Q_OBJECT
    public :
        SyncRequest( const Nepomuk::Sync::ChangeLog& log );
        SyncRequest();
        virtual ~SyncRequest();

        /**
        * Converts the contents of the LogFile into the MergeHash
        * The LogFile can get rather large, so this could be a time consuming
        * process.
        */
        void load();

        /**
        * Indicates if all the statements from the LogFile have been merged
        */
        bool done() const;

        /**
        * Unique Id of this mergeRequest
        */
        int id() const;

        /**
        * This is the main function that does half the work. It iterates through all
        * the resources in MergeHash, and foreach property and object, it resolves
        * conflicts and decides what statement should be chosen.
        * \sa resolveSingleCardinality resolveMultipleCardinality
        */
        void merge();
        
        int size() const;

    Q_SIGNALS:
        /**
        * Emits the percent completed when performing the actual merging. This does not
        * include conflict resolution.
        * 1 <= percent <= 100
        */
        void completed( int percent );

        void multipleMerge( const QString & uri, const QString & prop );

    private:
        class Private;
        Private *d;

        /**
         * Resolve conflicts for properties with multiple cardinalities. It works by locally
         * making all the changes in the ResourceStuct.
         */
        void resolveMultipleCardinality( const QList<Sync::ChangeLogRecord >& theirRecords, const QList<Sync::ChangeLogRecord >& ownRecords );
        
        /**
         * Same as multiple Cardinality resolution, but a lot faster. It figures out which statement
         * should be present by looking at the max time stamp.
         */
        void resolveSingleCardinality( const QList< Nepomuk::Sync::ChangeLogRecord >& theirRecords, const QList< Nepomuk::Sync::ChangeLogRecord >& ownRecords );
    };
}


#endif // SYNCREQUEST_H
