/* This file is part of the KDE Project
   Copyright (c) 2009 Daniel Winter <dw@danielwinter.de>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Lesser General Public
    License as published by the Free Software Foundation; either
    version 2.1 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public
    License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 Collects signals from different filesystemshandlers and forwards the
 signal with the highest priority after a given delay.

 If a filesystem has the set max priority the signal is forwared without
 a delay and further signals for the same mountpoint are blocked for the
 delay time.

 Delay should be a little longer than the slowest filesystemhandler is expected
 to be slower than any other same filesystem reporting handler.
*/

#ifndef MOUNT_SIGNAL_FILTER_H
#define MOUNT_SIGNAL_FILTER_H

#include <QObject>
#include <QPair>
#include <QString>

#include <KDebug>

#include "filesystem.h"

class QSignalMapper;
class QTimer;

namespace Nepomuk
{

class MountSignalFilter : public QObject
{
    Q_OBJECT

public:
    // delay in msec
    MountSignalFilter( QObject *parent, int delay, int maxPriority );

private:
    void doSlotFilesystem( const Filesystem& fs, QSignalMapper* mapper, QHash<QString, QPair< Filesystem, bool > >& mountList, bool mounted );
    void doSignalTimerForMount( const QString& mountpoint, QSignalMapper* mapper, QHash<QString, QPair< Filesystem, bool > >& mountList, bool mounted );


private:
    int m_delay;
    int m_maxPriority;
    QSignalMapper *m_mountSignalMapper;
    QSignalMapper *m_unmountSignalMapper;
    QHash<QString, QPair< Filesystem, bool > > m_mountList;
    QHash<QString, QPair< Filesystem, bool > > m_unmountList;
    QHash<QString, QTimer*> m_timerHash;

public Q_SLOTS:
    void slotFilesystemMounted( Filesystem fs );
    void slotFilesystemUnmounted( Filesystem fs );

private Q_SLOTS:
    void slotTimerForMountElapsed( const QString& mountpoint );
    void slotTimerForUnmountElapsed( const QString& mountpoint );

Q_SIGNALS:
    void filesystemMounted( Filesystem f );
    void filesystemUnmounted( Filesystem f );
};

}

#endif // MOUNT_SIGNAL_FILTER_H
