/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WiTHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "emailresourceeditplugin.h"

#include "../src/propertyeditwidget.h"
#include "../src/nepomukdataaccess.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QPalette>
#include <QColor>
#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QFormLayout>
#include <QGridLayout>
#include <QSet>
#include <QUrl>
#include <QLabel>
#include <KComboBox>
#include <QMap>

#include <Nepomuk/Types/Property>
#include <Nepomuk/Variant>
#include <Nepomuk/Resource>
#include <Soprano/Vocabulary/RDFS>
#include <nmo.h>
#include <nco.h>

#include <KPluginFactory>
#include <KPluginLoader>
#include <KDebug>
#include <KLocalizedString>
#include <KAction>
#include <KMessageBox>


K_PLUGIN_FACTORY(EmailResourceEditPluginFactory, registerPlugin<Nepomuk::EmailResourceEditPlugin>();)
K_EXPORT_PLUGIN(EmailResourceEditPluginFactory("nepomuk_emailresourceeditplugin"))

class Nepomuk::EmailResourceEditPlugin::Private
{
public:
    Private() {
    }

    QList<PropertyEditWidget*> propertyEditWidgets;
    KComboBox *stateComboBox;
    QMap<QString,QString> stateStrings;
};

Nepomuk::EmailResourceEditPlugin::EmailResourceEditPlugin(QWidget *parent, const QList<QVariant>& ) :
        AbstractResourceEditPlugin(parent),
        d( new Private() )
{
}

Nepomuk::EmailResourceEditPlugin::~EmailResourceEditPlugin()
{
    delete d;
}

void Nepomuk::EmailResourceEditPlugin::initGui()
{    

    //IDEA: only display stuff, don't allow editing. It gives greater freedom (e.g. displaying the name of the sender nco:Contact)

    if (layout()) {
        layout()->deleteLater();
    }

    //add main layout
    QVBoxLayout *mainLayout = new QVBoxLayout();
    mainLayout->setContentsMargins(0, 0, 0, 0);

    //from: name addres
    //subject: text
    //to: name address
    //body...................


    //add from, subject, to and date
    QFormLayout *formLayout = new QFormLayout;

    //TODO? if Sender is defined, use that too
    Nepomuk::Resource fromContact = resource().property(Nepomuk::Vocabulary::NMO::messageFrom()).toResource();
    QString fromName = fromContact.property(Nepomuk::Vocabulary::NCO::fullname()).toString();
    QString fromAddress = fromContact.property(Nepomuk::Vocabulary::NCO::hasEmailAddress()).toResource().property(Nepomuk::Vocabulary::NCO::emailAddress()).toString();

    QLabel *labelFrom = new QLabel(fromName.isEmpty() ? fromAddress : fromName + "<" + fromAddress + ">");
    QLabel *labelSubject = new QLabel(resource().property(Nepomuk::Vocabulary::NMO::messageSubject()).toString());

    Nepomuk::Resource toContact = resource().property(Nepomuk::Vocabulary::NMO::emailTo()).toResource();
    QString toName = toContact.property(Nepomuk::Vocabulary::NCO::fullname()).toString();
    QString toAddress = toContact.property(Nepomuk::Vocabulary::NCO::hasEmailAddress()).toResource().property(Nepomuk::Vocabulary::NCO::emailAddress()).toString();

    QLabel *labelTo = new QLabel(toName.isEmpty() ? toAddress : toName + "<" + toAddress + ">");
    QString date;
    if(resource().property(Nepomuk::Vocabulary::NMO::receivedDate()).isValid()) {
        date = KGlobal::locale()->formatDateTime(resource().property(Nepomuk::Vocabulary::NMO::receivedDate()).toDateTime());
    }
    else if (resource().property(Nepomuk::Vocabulary::NMO::sentDate()).isValid()){
        date = KGlobal::locale()->formatDateTime(resource().property(Nepomuk::Vocabulary::NMO::sentDate()).toDateTime());
    }
    QLabel *labelDate = new QLabel(date);

    formLayout->addRow(i18n("From:"), labelFrom);
    formLayout->addRow(i18n("Subject:"), labelSubject);
    formLayout->addRow(i18n("To:"), labelTo);
    formLayout->addRow(i18n("Date:"), labelDate);
    mainLayout->addLayout(formLayout);

    QLabel *labelBody = new QLabel();
    //TODO: have a More... button, maybe
    labelBody->setText(resource().property(Nepomuk::Vocabulary::NMO::plainTextMessageContent()).toString());
    mainLayout->addWidget(labelBody);

    //set layout and focus policy
    setLayout(mainLayout);
}


void Nepomuk::EmailResourceEditPlugin::setupActions()
{
    AbstractResourceEditPlugin::setupActions();

    KAction *reply = new KAction(KIcon("mail-reply-custom"), i18n("Reply to email"), this);
    connect(reply, SIGNAL(triggered()), this, SLOT(replyToEmail()));
    addAction(reply);

    KAction *forward = new KAction(KIcon("mail-forward"), i18n("Forward email"), this);
    connect(forward, SIGNAL(triggered()), this, SLOT(forwardEmail()));
    addAction(forward);
}

void Nepomuk::EmailResourceEditPlugin::replyToEmail()
{
    //TODO: implement this

    kDebug() << "*** replyToEmail: not implemented";
}

void Nepomuk::EmailResourceEditPlugin::forwardEmail()
{
    //TODO: implement this

    kDebug() << "*** forwardEmail: not implemented";
}
