/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "querybuilderwidget.h"
#include "searchwidget_p.h"
#include "searchlineedit.h"

#include <KDebug>

#include <Nepomuk/Query/Query>
#include <Nepomuk/Query/QueryParser>
#include <Nepomuk/Query/Term>
#include <Nepomuk/Resource>


class Nepomuk::Query::QueryBuilderWidget::QueryBuilderWidgetPrivate :
    public Nepomuk::Query::SearchWidget::SearchWidgetPrivate
{
};

Nepomuk::Query::QueryBuilderWidget::QueryBuilderWidget(QWidget *parent)
    : SearchWidget(new QueryBuilderWidgetPrivate, parent)
{
    Q_D(QueryBuilderWidget);
    connect(d->m_queryEdit, SIGNAL(queryChanged(Nepomuk::Query::Query)),
            this, SLOT(emitQueryChanged()));
    connect(d->m_facetWidget, SIGNAL(facetsChanged()),
            this, SLOT(emitQueryChanged()));
}

Nepomuk::Query::QueryBuilderWidget::~QueryBuilderWidget()
{
    //no need to delete d-pointer (base class does it)
}

Nepomuk::Query::Query Nepomuk::Query::QueryBuilderWidget::currentQuery() const
{
    Q_D(const QueryBuilderWidget);

    return Nepomuk::Query::Query(d->currentQuery());
}

/*
void Nepomuk::Query::QueryBuilderWidget::setLineEditQuery(const Nepomuk::Query::Query &query)
{
    Q_D(QueryBuilderWidget);
    d->queryEdit->setQuery(query);

}

Nepomuk::Query::Query Nepomuk::Query::QueryBuilderWidget::lineEditQuery() const
{
    Q_D(const QueryBuilderWidget);
    return d->queryEdit->query();
}

void Nepomuk::Query::QueryBuilderWidget::setFacetsWidgetQuery(const Nepomuk::Query::Query &query)
{
    Q_D(QueryBuilderWidget);
    d->facetsWidget->setQuery(query);

}

Nepomuk::Query::Query Nepomuk::Query::QueryBuilderWidget::facetsWidgetQuery() const
{
    Q_D(const QueryBuilderWidget);
    return d->facetsWidget->constructQuery();
}
*/

void Nepomuk::Query::QueryBuilderWidget::emitQueryChanged()
{
    emit queryChanged(currentQuery());
}

#include "querybuilderwidget.moc"
