/*
Copyright (c) 2010 Oszkar Ambrus <aoszkar@gmail.com>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) version 3, or any
later version accepted by the membership of KDE e.V. (or its
successor approved by the membership of KDE e.V.), which shall
act as a proxy defined in Section 6 of version 3 of the license.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef RESOURCECREATEANDSEARCHWIDGET_H
#define RESOURCECREATEANDSEARCHWIDGET_H

#include <QWidget>

#include "nepomuksearchgui_export.h"

class QMouseEvent;
class QEvent;

namespace Nepomuk
{
    namespace Types
    {
        class Class;
    }
    namespace Query
    {
        class Result;
    }
    class Resource;

    /**
     *  A widget to create or search for Resources, using NewResourceDialog and ResourceSearchDialog
     */

    class NEPOMUKSEARCHGUI_EXPORT ResourceCreateAndSearchWidget : public QWidget
    {
        Q_OBJECT

    public:
        /**
         * Creates a new ResourceCreateAndSearchWidget
         */
        ResourceCreateAndSearchWidget(QWidget *parent = 0);
        ~ResourceCreateAndSearchWidget();

        /**
         * Sets the type of resource to @p type
         */
        void setType(const Types::Class &type);

        /**
         * @return the current resource
         */
        Resource resource() const;

    public Q_SLOTS:
        /**
         * Set the current resource to @p resource
         */
        void setResource(const Nepomuk::Resource &resource);

    Q_SIGNALS:
        /**
         * The signal is emitted when the current resource is changed
         */
        void valueChanged();

    private:
        void changeEvent (QEvent * event);

        class Private;
        Private * const d;

        //class ResourceSearchLineEdit;

    private Q_SLOTS:
        /**
         * Shows the current resource in a ResourceEditDialog
         */
        void showResource();
        /**
         * Displays a dialog (using NewResourceWidget) for creating a new resource,
         * sets the current resource to the one returned by the dialog
         * and emits the valueChanged() signal
         */

        void createResource();

        /**
         * Sets the current resource to the one of the ResourceSearchWidget
         * and emits the valueChanged() signal
         */
        void searchResource();

        /**
         * Copies the label of the current resource to the clipboard
         */
        void copyLabelToClipboard();

        /**
         * Copies the URI of the current resource to the clipboard
         */
        void copyUriToClipboard();

        /**
          * Updates the label text
          */
        void updateLabel();
    };
}

#endif // RESOURCECREATEANDSEARCHWIDGET_H
