/*
   This file is part of the Nepomuk KDE project.
   Copyright (C) 2007 Sebastian Trueg <trueg@kde.org>

   This library is free software; you can redistribute it and/or
   modify it under the terms of the GNU Library General Public
   License version 2 as published by the Free Software Foundation.

   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Library General Public License for more details.

   You should have received a copy of the GNU Library General Public License
   along with this library; see the file COPYING.LIB.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
 */

#include "plasmaresourcewidget.h"
#include "abstractresourceguiitem.h"
#include "resourceguiitemfactory.h"

#include <nepomuk/resource.h>

#include <QtGui/QPainter>
#include <QtGui/QApplication>


class Nepomuk::PlasmaResourceWidget::Private
{
public:
    Resource resource;

    ResourceGuiItemFactory factory;

    AbstractResourceGuiItem::DisplayMode displayMode;
};


Nepomuk::PlasmaResourceWidget::PlasmaResourceWidget( QGraphicsItem* parent, QObject* parentObject )
    : Plasma::Widget( parent, parentObject ),
      d( new Private() )
{
}


Nepomuk::PlasmaResourceWidget::~PlasmaResourceWidget()
{
    delete d;
}


QSizeF Nepomuk::PlasmaResourceWidget::sizeHint() const
{
    QStyleOption option;
    option.direction = QApplication::layoutDirection();
    option.fontMetrics = QApplication::fontMetrics();
    option.palette = QApplication::palette();
    return d->factory.guiItemForType( d->resource.resourceType() )->sizeHint( d->displayMode, option, d->resource.resourceUri() );
}


void Nepomuk::PlasmaResourceWidget::paintWidget( QPainter* painter, const QStyleOptionGraphicsItem* option, QWidget* widget )
{
    Q_UNUSED( widget );

    QStyleOptionGraphicsItem o;
    if ( option )
        o = *option;

    painter->fillRect( o.rect, Qt::white );
    d->factory.guiItemForType( d->resource.resourceType() )->paint( painter, d->displayMode, o, d->resource.resourceUri() );
}


void Nepomuk::PlasmaResourceWidget::setResource( const QUrl& uri )
{
    d->resource = Resource( uri );
    update();
}


void Nepomuk::PlasmaResourceWidget::setDisplayMode( AbstractResourceGuiItem::DisplayMode mode )
{
    d->displayMode = mode;
    update();
}


Nepomuk::AbstractResourceGuiItem::DisplayMode Nepomuk::PlasmaResourceWidget::displayMode() const
{
    return d->displayMode;
}


QUrl Nepomuk::PlasmaResourceWidget::resource() const
{
    return d->resource.resourceUri();
}

#include "plasmaresourcewidget.moc"
