#!/usr/bin/env python
"""
This test should be run in a dedicated KDE session.
"""

import sys
import os
import os.path
import unittest
import time
import dbus
from PyQt4 import QtCore
from PyKDE4 import kdecore, soprano, nepomuk

sys.path.append(os.path.dirname(os.path.realpath(__file__)) + '/../core')
import nepomuktest


class StrigiTestCase(nepomuktest.NepomukServiceTestBase):
    def strigiDBusInterface(self):
        "Return the DBus proxy interface to the strigi service"
        return self.serviceDBusInterface('nepomukstrigiservice', 'org.kde.nepomuk.Strigi')


    def waitForStrigiIdle(self):
        "Sleeps until the strigi service reports an idle status"
        try:
            strigi = self.strigiDBusInterface()
            while strigi.isIndexing():
                time.sleep(2)
            return True
        except dbus.exceptions.DBusException:
            return False
        

    def setUpTestCase(self):
        self.setNecessaryServices( [ 'nepomukstrigiservice' ] )
        self.nepomukConfig.setStrigiIndexDirs([ self.getTestDataBaseDir() ])


    def testIndexFolder(self):
        "Test indexing a single folder of different files"

        testDir = self.getTestDataBaseDir()
        testFiles = [ f for f in [ os.path.join(testDir,e) for e in os.listdir(testDir) ] if not QtCore.QFileInfo(f).isHidden() ]

        # index the test dir
        strigi = self.strigiDBusInterface()
        strigi.indexFolder(testDir,True)
        self.assert_(self.waitForStrigiIdle())

        # remember the resource URIs
        testFileResourceUriMap = {}

        urlQuery = 'select ?r where { ?r <http://www.semanticdesktop.org/ontologies/2007/01/19/nie#url> %s . }'
        
        # now check if the data has been properly generated
        for path in testFiles:
            query = urlQuery % soprano.Soprano.Node.resourceToN3(kdecore.KUrl(path))
            nodes = nepomuk.Nepomuk.ResourceManager.instance().mainModel().executeQuery( query, soprano.Soprano.Query.QueryLanguageSparql ).iterateBindings('r').allNodes()

            # there should only be one
            self.assertEqual(len(nodes),1)

            # remember it for later comparison
            testFileResourceUriMap[path] = nodes[0].uri()

            # and it should have a nepomuk:/ URI
            self.assert_(nodes[0].toString().startsWith('nepomuk:/'))

            # libnepomuk should find the same URI
            self.assertEqual(nodes[0].uri(),nepomuk.Nepomuk.Resource(kdecore.KUrl(path)).resourceUri())

        # re-index the test dir
        strigi = self.strigiDBusInterface()
        strigi.indexFolder(testDir,True)
        self.assert_(self.waitForStrigiIdle())

        # make sure strigi does not change the resource URIs
        for path in testFiles:
            query = urlQuery % soprano.Soprano.Node.resourceToN3(kdecore.KUrl(path))
            nodes = nepomuk.Nepomuk.ResourceManager.instance().mainModel().executeQuery( query, soprano.Soprano.Query.QueryLanguageSparql ).iterateBindings('r').allNodes()

            # there should only be one
            self.assertEqual(len(nodes),1)

            # compare to cached URI
            self.assertEqual(nodes[0].uri(),testFileResourceUriMap[path])
            

if __name__ == '__main__':
    nepomuktest.nepomukUnittestMain()
    
