/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef ANNOTATED_MARBLE_WIDGET_H_
#define ANNOTATED_MARBLE_WIDGET_H_

#include <QtCore/QMap>
#include <QtCore/QString>
#include <QtCore/QVariant>
#include <QtGui/QGraphicsWidget>
#include <QtCore/QMutex>

#include <marble/MarbleWidget.h>
#include <marble/GeoPainter.h>
#include <marble/GeoDataPoint.h>

/**
 * A MarbleWidget that draws annotations for all contacts
 */
class AnnotatedMarbleWidget: public Marble::MarbleWidget
{
Q_OBJECT

	struct Annotation
	{
		QString text;
		Marble::GeoDataPoint point;
	};

public:
	/**
	 * Constructor
	 * @param parent Parent widget
	 */
	AnnotatedMarbleWidget(QWidget *parent = 0)						;

	/** Destructor */
	virtual~ AnnotatedMarbleWidget();

	/**
	 * Properties of some contact have changed
	 * @param id Contact id
	 * @param properties New properties
	 *
	 * @todo: This should be done in MarbleContactsWidget instead. An updateAnnotation()
	 * method should be enough here.
	 */
	void propertiesChanged(const QString &id, const QVariantMap &properties);

	/**
	 * Toggle whether a change of a contact's location should trigger a
	 * recentering of the map. Default is true.
	 * @param enabled True iff map should follow contact's location changes
	 */
	void setRecenterOnLocationChange(bool enabled);

	/**
	 * Reimplemented from MarbleWidget to draw contact annotations in the map
	 */
	void customPaint(Marble::GeoPainter *painter);

protected:
	/** Reimplemented from QWidget */
	void resizeEvent ( QResizeEvent * event );

private:
	/** Annotations to be drawn */
	QMap<QString, Annotation> m_annotations;

	/** Stores whether location changes trigger map recentering */
	bool m_recenterOnLocationChange;
	
	/** Prevents concurrent access to m_annotations */
	QMutex m_annotationsMutex;
};

#endif // ANNOTATED_MARBLE_WIDGET_H_
