/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef CONTACT_WIDGET_H_
#define CONTACT_WIDGET_H_

#include <QtGui/QGraphicsWidget>

// Forward declarations
class ContactWidgetPrivate;

/**
 * Widget to display information about a contact and actions to communicate with it.
 */
class ContactWidget: public QGraphicsWidget
{
Q_OBJECT

public:
	/**
	 * Constructor
	 * @param cells Cells to display
	 * @param properties Initial properties used during cell initialisation
	 * @param parent Parent widget
	 */
	ContactWidget(const QStringList &cells, const QVariantMap &properties, QGraphicsItem *parent = 0)	;

	/** Destructor */
	virtual ~ContactWidget();

	/** Update properties for all cells
	 * @param properties Contacts properties
	 */
	void propertiesChanged(const QVariantMap &properties);

private:
	/** d-pointer */
	ContactWidgetPrivate * const d;
};

#endif // CONTACT_WIDGET_H_
