#include "../main.h"
#include "Ui_CpuFreq.h"
#include <Plasma/PaintUtils>
#include <QMessageBox>
#include <KMessageBox>

QString testAvailability(Plasma::Applet *app) {
	// If /sys/devices/system/cpu/cpu%s/cpufreq print error message asking user to enable cpufreq modules
	// if !cpufreq-set print error asking user to install cpufreq-utils package
	// if can't execute cpufreq-set ask user to set permissions
	// If policykit were enabled the dependency on cpufreq-utils could be removed more easily only requiring the kernel modules
	// If only regular users could (using policykit) echo to the cpufreq system nodes
	QString str;
	if( (str  = plasma_tools::runCommand("which cpufreq-set")) == QString("")) {
		QString Error = i18n("cpufreq-set from cpufreq-utils was not found. Please verify you have the cpufreq-utils package installed.");
		return Error;
	}
	if(plasma_tools::runCommand("cpufreq-set").contains("not found")) {
		QString Error = i18n("Error, cannot execute cpufreq-set. please run: chmod +s %1", str);
		return Error;
	}
	std::ifstream file("/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor");
	file.seekg (0, ios::end);
        int size = file.tellg();

	if(size == -1) {
		QString Error = i18n("Error, You don't appear to be running the cpufreq modules, please see your documentation on how to enable cpu frequency scaling for your processor.");
		QString title = "Not running cpufreq modules";
		return Error;
	} else {
		file.close();
	}

	return QString();
}

PlasmaCpufreq::PlasmaCpufreq(QObject *parent, const QVariantList &args) : Plasma::Applet(parent, args)
{
	ui = new Ui::CpuFreq(this);
	clickedyet = false;

	setHasConfigurationInterface(true);
	cpu = 0; // The only default. Everything else set by cpufreq

	QString error = testAvailability(this);
	if(error.isEmpty()) {
		governors = getAvailableGovernors();
		frequencies = getAvailableFrequencies();
		governor = getCurrentGovernor();
		frequency = getCurrentFrequency();
		cpus = getAvailableCpus();
		cpu = getCurrentCpu();
	
		QTimer *updateTimer = new QTimer(parent);
		connect(updateTimer, SIGNAL(timeout()), this, SLOT(updateme()));
		updateTimer->start(1000);
	} else {
		setFailedToLaunch(true, error);
	}
}
void PlasmaCpufreq::mousePressEvent(QGraphicsSceneMouseEvent *event) {
	if(event->buttons() == Qt::LeftButton) {
		if(!ui) {
			return;
		}
		if(ui->isVisible()) {
			ui->hide();
			return;
		}
		if (!clickedyet) {

			ui->move(popupPosition(ui->size()));
			clickedyet = true;
//                                        # Only run this the first time the window is opened, after that, use the values stored on the window
//                                        # This allows custom positions and resizing - when plasma allows it ...
		}
		//         # Show the window (not maximized and restore from the task tray if it's minimized)
		ui->showNormal();
//		# Raise it above other windows
		ui->raise();
//		# Give it activation
		ui->activateWindow();
	}

}
PlasmaCpufreq::~PlasmaCpufreq() {
	delete ui;
}

QRect preparePainter(QPainter *p, const QRect &rect, const QFont &font, const QString &text, bool singleline)
{
    QRect tmpRect;
    QFont tmpFont = font;
    bool first = true;

    // Starting with the given font, decrease its size until it'll fit in the
    // given rect allowing wrapping where possible
    do {
        if (first) {
            first = false;
        } else  {
            tmpFont.setPointSize(qMax(KGlobalSettings::smallestReadableFont().pointSize(), tmpFont.pointSize() - 1));
        }

        QFontMetrics fm(tmpFont);
        int flags = Qt::TextSingleLine ;

        tmpRect = fm.boundingRect(rect, flags, text);
    } while (tmpFont.pointSize() > KGlobalSettings::smallestReadableFont().pointSize() &&
             (tmpRect.width() > rect.width() || tmpRect.height() > rect.height()));

    p->setFont(tmpFont);
    return tmpRect;
}
void PlasmaCpufreq::paintInterface(QPainter *p,
                const QStyleOptionGraphicsItem *option,
                const QRect& rect) 
{
                int margin = 4;
		p->setPen(QPen(Qt::NoPen));
		p->setRenderHint(QPainter::SmoothPixmapTransform);
		p->setRenderHint(QPainter::Antialiasing);

		QRectF nrect(rect.adjusted(-margin, -margin, margin, margin));
                p->drawPath(Plasma::PaintUtils::roundedRectangle(nrect, margin));

                p->setPen( Plasma::Theme::defaultTheme()->color(Plasma::Theme::TextColor));

                QFont plainFont = KGlobalSettings::generalFont();
		plainFont.setPointSizeF(qMax(rect.height(), KGlobalSettings::smallestReadableFont().pointSize()));
                preparePainter(p, rect, plainFont, Text, true);
		
		p->drawText(rect, Qt::TextWordWrap | Qt::AlignHCenter, Text);
                p->save();
                p->restore();

}
void PlasmaCpufreq::init(){}
QGraphicsWidget *PlasmaCpufreq::graphicsWidget() {
	return uiProxy;
}
void PlasmaCpufreq::updateme() {
	Text = getCurrentFrequency();
	update();
} // timeout function, called every 1s
QList<QString> PlasmaCpufreq::getAvailableFrequencies() {
	QString cmd = "/sys/devices/system/cpu/cpu%s/cpufreq/scaling_available_frequencies";
	QString freqs = plasma_tools::FrequencyFileReader::self()->readFile(cmd, cpu);
	QList<QString> freqList = freqs.simplified().split(" ");
	QList<QString>::iterator iter = freqList.begin();
	QList<plasma_tools::FrequencyMap> freqMap;
	for(;iter != freqList.end();iter++ ){
		QString &freq = *iter;
		QString orig = freq;
		if(freq.length() > 6) {
			freq = freq.left( freq.length()-4);
			freq.insert(1, ".");
			freq = freq + "GHz";
		} else {
			if(freq.length() > 4) {
				freq = freq.left( freq.length()-3);
				freq = freq + "MHz";
			}
		}
		plasma_tools::FrequencyMap element;
		element.orig = orig;
		element.modified = freq;
		freqMap.push_back(element);
	}
	ui->fillFreqListView(freqMap, freqList);
	return frequencies = freqList;
}
QList<QString> PlasmaCpufreq::getAvailableGovernors() {
	QString cmd = "/sys/devices/system/cpu/cpu%s/cpufreq/scaling_available_governors";
	QString govs = plasma_tools::FrequencyFileReader::self()->readFile(cmd, cpu);
	QList<QString> govList = govs.simplified().split(" ");
	QList<QString>::iterator iter = govList.begin();
	for(;iter != govList.end();iter++ ){
		if(*iter == QString("userspace") ) {
			govList.erase(iter);
			break;
		}
	}
	ui->fillGovListView(govList);
	return governors = govList;
}
int PlasmaCpufreq::getAvailableCpus() // returns the number of cpu's
{
	QString str = plasma_tools::runCommand("ls -d /sys/devices/system/cpu/cpu[0-9]*");
	QList<QString> list = str.simplified().split(" ");
	ui->fillCpuListView(list);
	return cpus = list.count()-1;
}
QString PlasmaCpufreq::getCurrentFrequency()
{
	QString cmd = "/sys/devices/system/cpu/cpu%s/cpufreq/scaling_cur_freq";
	QString freq = plasma_tools::FrequencyFileReader::self()->readFile(cmd, cpu);
	freq = freq.simplified();
	if(freq.length() > 6) {
		freq = freq.left(freq.length()-4);
			freq.insert(1, ".");
			freq = freq + "GHz";
	} else {
		if(freq.length() > 4) {
			freq = freq.left( freq.length()-3);
			freq = freq + "MHz";
		}
	}
	return frequency=freq;
}
QString PlasmaCpufreq::getCurrentGovernor()
{
	QString cmd = "/sys/devices/system/cpu/cpu%s/cpufreq/scaling_governor";
	QString gov = plasma_tools::FrequencyFileReader::self()->readFile(cmd, cpu);
	return governor=gov;
}
int PlasmaCpufreq::getCurrentCpu()
{
	return cpu;
} // default to zero,  user selected.

K_EXPORT_PLASMA_APPLET(cpufrequencyselector, PlasmaCpufreq)
#include "../main.moc"
