/*****************************************************************************
 *   Copyright (C) 2009-2010 by Matthieu Gallien <matthieu_gallien@yahoo.fr> *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License, or       *
 *   (at your option) any later version.                                     *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .          *
 *****************************************************************************/

#ifndef TASKS_MODEL_HEADER
#define TASKS_MODEL_HEADER

#include <Plasma/DataEngine>
#include <QAbstractListModel>
#include <QModelIndex>

class FlatTasksModel : public QAbstractListModel
{
    Q_OBJECT
public:
    FlatTasksModel(Plasma::DataEngine *tasksEngine,
                   Plasma::DataEngine *statusNotifierItemEngine,
                   QObject *parent = 0);

    virtual ~FlatTasksModel();

    QVariant data(const QModelIndex &index, int role = Qt::DisplayRole) const;

    int rowCount(const QModelIndex &index = QModelIndex()) const;

protected slots:

    void taskAdded(const QString &taskName);
    void taskRemoved(const QString &taskName);

    void dataUpdated(const QString &taskName, const Plasma::DataEngine::Data &taskData);

    void statusNotifierItemAdded(const QString &itemName);
    void statusNotifierItemRemoved(const QString &itemName);

protected:

    typedef QPair<QString, QStringList> itemInfo_type;

    void addItemFromWindowID(const QString &windowId,
                             const Plasma::DataEngine::Data &taskData);

    void removeItemFromWindowID(const QString &windowId);

    void addItemFromStatusNotifierItem(const QString &itemName,
                                       const Plasma::DataEngine::Data &itemData);

    void removeItemFromStatusNotifierItem(const QString &itemName);

    void updateCachedIndex(int removedIndex);

    void updateBackgroundPrefix(const QString &taskName, const Plasma::DataEngine::Data &taskData);

    bool statusNotifierItemHaveMatchingTasks(const QString &itemName) const;

    bool statusNotifierItemIsApplication(const QString &itemName) const;

    bool statusNotifierItemIsActive(const QString &itemName) const;

    QVariant createContextMenuForStatusItem(const QString &statusItemName) const;

    QList<QVariant> createContextMenuForTask(const QString &taskName) const;

    QVariantMap createActivateActionForStatusItem(const QString &statusItemName) const;

    QVariantMap createActivateActionForTask(const QString &taskName) const;

    QVariantMap createSecondaryActivateActionForStatusItem(const QString &statusItemName) const;

private:

    void debugPrint() const;

    Plasma::DataEngine *m_tasksEngine;
    Plasma::DataEngine *m_statusNotifierItemEngine;

    QHash<QString, Plasma::DataEngine::Data> m_tasksData;
    QHash<QString, Plasma::DataEngine::Data> m_statusNotifierItemsData;
    QList<itemInfo_type> m_allItems;

    QHash<QString, int> m_indexFromTaskName;
    QHash<QString, int> m_indexFromStatusNotifierItemName;

    int m_numberOfDesktops;
    QList<QVariant> m_desktopNames;
};

#endif
