/***************************************************************************
 *   Copyright (C) 2009 by Björn Ruberg <bjoern@ruberg-wegener.de>         *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "KopeteExtender.h"
#include <QGraphicsScene>
#include "chatdialog.h"

KopeteExtender::KopeteExtender(QObject *parent, const QVariantList &args)  :
	Plasma::PopupApplet(parent, args), m_contactslist(0) {
	//etAspectRatioMode(Plasma::IgnoreAspectRatio);
	setPopupIcon("kopete");
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
        
	m_contactslist = new ContactsList(this);
	m_interface = new org::kde::Kopete("org.kde.kopete", "/Kopete", QDBusConnection::sessionBus());

	updateContacts();
	updateList("online");

	connect(m_interface, SIGNAL(contactChanged(const QString &)),
		    this, SLOT(updateContact(const QString &)));
	//connect(this, SIGNAL(sizeHintChanged()), this, SLOT(sizeChanged()));
}


KopeteExtender::~KopeteExtender() {
}

void KopeteExtender::init() {
    /*if(!extender()->hasItem("contactlist")) {
	Plasma::ExtenderItem *eItem = new Plasma::ExtenderItem(extender());
	eItem->setName("contactlist");
	initExtenderItem(eItem);
    }*/


}


QGraphicsWidget* KopeteExtender::graphicsWidget(){
    if(!m_contactslist){
	m_contactslist = new ContactsList(this);
    }
    return m_contactslist;
}

void KopeteExtender::initExtenderItem(Plasma::ExtenderItem *item){
    if(!m_contactslist){
	m_contactslist = new ContactsList(this);
    }

    item->setWidget(m_contactslist);
    item->setTitle("contacts");
}

QMap<QString,Contact*> KopeteExtender::updateContacts() {

    QDBusReply < QStringList > contactList = m_interface->contacts();
    if ( ! contactList.isValid() ) { return m_contacts; }

    foreach (const QString& contact, contactList.value()) {
	if (m_contacts.contains(contact)){
	    m_contacts[contact]->update();
	}
	else {
	    m_contacts[contact] = new Contact ( contact, m_interface );
	}
    }

    return m_contacts;
}

void KopeteExtender::updateList(const QString &status) {
    QDBusReply < QStringList > contactList = m_interface->contactsByFilter(status);
    QList<Contact*> onlineContacts;

    foreach (const QString& contact, contactList.value()) {
	onlineContacts << m_contacts[contact];
    }

    m_contactslist->updateList( onlineContacts );
}

void KopeteExtender::updateContact(const QString &contact) {
    if(m_contacts.contains(contact)){
	m_contacts[contact]->update();
    }
    updateList("online");
}


void KopeteExtender::openChat(Contact *contact){
    new ChatDialog(this, contact);
}



// This is the command that links your applet to the .desktop file
K_EXPORT_PLASMA_APPLET(kopete_extender, KopeteExtender)
