/***************************************************************************
 *   Copyright (C) 2009 by Björn Ruberg <bjoern@ruberg-wegener.de          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "contact.h"
#include "contactlistitem.h"
#include <QGraphicsGridLayout>
#include <KIcon>


Contact::Contact(QString contact, org::kde::Kopete* interface) : QObject(),avatar(0),m_listItem(0)
{
    m_interface = interface;
    identity = contact;

    update();
}

Contact::~Contact(){
    if (avatar) {delete avatar;}
    if (m_listItem) {delete m_listItem;}
}

void Contact::update() {
    QDBusReply < QVariantMap > properties = m_interface->contactProperties(identity);
    QDBusReply < bool > online = m_interface->isContactOnline(identity);

    if ( ! properties.isValid() || ! online.isValid() ) {return;}

    displayName = properties.value().value("display_name").toString();
    statusMessage = properties.value().value("status_message").toString();
    status = properties.value().value("status").toString();
    picture = properties.value().value("picture").toString();
    
    if ( picture.size() == 0 ) {
	picture = "user-" + status.toLower();
    }

    isOnline = online.value();

    avatar = new KIcon(picture);

    if ( m_listItem ){
	m_listItem->setIcon(avatar);
	m_listItem->setNick(displayName);
    }
}

QString Contact::getDisplayName(){
    return displayName;
}

QString Contact::getStatusMessage(){
    return statusMessage.isEmpty() ? status : statusMessage;
}

KIcon* Contact::getAvatar(){
    return avatar;
}

ContactListItem* Contact::getListItem(QGraphicsWidget* parent){
    if( m_listItem ) {
	return m_listItem;
    }
    else {
	m_listItem = new ContactListItem(parent, this);
	return m_listItem;
    }
}
