/*
    This file is part of the Program Menu plasmoid, part of the KDE project.

    Copyright (c) 2002 Siegfried Nijssen <snijssen@liacs.nl>
    Copyright (c) 2003 Lubos Lunak <l.lunak@suse.cz>
    Copyright 2008 Friedrich W. H. Kossebau <kossebau@kde.org>

    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:

    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL THE
    AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN
    AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
    CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
*/

#ifndef MENUBAREMBEDCONTAINER_H
#define MENUBAREMBEDCONTAINER_H


// KDE
#include <KDebug>
// Qt
#include <QtCore/QVariant> // needed to avoid X11 #define's
#include <QtGui/QX11EmbedContainer>
// X11
#include <X11/Xlib.h>


class MenuBarEmbedContainer : public QX11EmbedContainer
{
  Q_OBJECT

  public:
    /**
     * Creates msg_type_atom and selection_atom, and returns the latter.
     */
    static Atom makeSelectionAtom();

  public:
    explicit MenuBarEmbedContainer( WId mainWindowId );
    virtual ~MenuBarEmbedContainer();

  public: // get
    /**
     * @returns the window ID for the handled window.
     */
    WId mainWindowId() const;

  public: // set
    /// overwrites QWidget::setMinimumSize()
    void setMinimumSize( int width, int height );
    /// convenience function
    void setMinimumSize( const QSize &size );

  Q_SIGNALS:
    void menuLost( MenuBarEmbedContainer *menuContainer );
    void sizeChanged( MenuBarEmbedContainer *menuContainer );

  protected: // QWidget API
    virtual bool x11Event( XEvent *event );

  protected:
    void sendSyntheticConfigureNotifyEvent() const;

  protected Q_SLOTS:
    /// emits menuLost signal
    void onClientClosed();
    /// emits menuLost signal
    void onError( QX11EmbedContainer::Error error );

  protected:
    WId mMainWindowId;
};


inline void MenuBarEmbedContainer::setMinimumSize( const QSize &size )
{ setMinimumSize( size.width(), size.height()); }

inline WId MenuBarEmbedContainer::mainWindowId() const { return mMainWindowId; }

#endif
