/***************************************************************************
 *   Copyright (C) 2008 by Sebastian Kuegler <sebas@kde.org>               *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "mid_control.h"
#include <limits.h>

// Qt
//#include <QPainter>
//#include <QBrush>
#include <QLabel>
#include <QSlider>

//#include <QGraphicsSceneHoverEvent>
//#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>

// KDE
//#include <KGlobalSettings>

// Plasma
#include <plasma/extender.h>
#include <plasma/extenderitem.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/slider.h>


MidControl::MidControl(QObject *parent, const QVariantList &args)
    : Plasma::PopupApplet(parent, args)
{
    setPopupIcon("player-volume");
    setAspectRatioMode(Plasma::Square );
    //setSizePolicy(QSizePolicy(QSizePolicy::Preferred, QSizePolicy::Preferred));
    //setBackgroundHints(NoBackground); // TODO: conditionally, pls.
    setMinimumSize(8, 8);
    setMaximumSize(INT_MAX, INT_MAX);
    setPopupIcon(QIcon());
    resize(22, 22);
}

void MidControl::init()
{
    extender()->setSizePolicy(QSizePolicy::MinimumExpanding, QSizePolicy::Fixed);
    Plasma::ExtenderItem *item = new Plasma::ExtenderItem(extender());
    setupExtenderItem(item);
    item->setName("brighness_volume");
}

MidControl::~MidControl ()
{
}


void MidControl::setupExtenderItem(Plasma::ExtenderItem *item)
{
    item->setTitle(i18n("Brightness and Volume"));

    QGraphicsWidget *controls = new QGraphicsWidget(item);
    QGraphicsLinearLayout *controlsLayout = new QGraphicsLinearLayout(controls);

    controlsLayout->setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);
    controlsLayout->setOrientation(Qt::Vertical);

    Plasma::Label *brightnessLabel = new Plasma::Label(controls);
    brightnessLabel->setText(i18n("Adjust Brightness"));
    brightnessLabel->nativeWidget()->setWordWrap(false);
    controlsLayout->addItem(brightnessLabel);

    Plasma::Slider *brightnessSlider = new Plasma::Slider(controls);
    brightnessSlider->setRange(0, 10);
    brightnessSlider->nativeWidget()->setTickInterval(2);
    brightnessSlider->setOrientation(Qt::Horizontal);
    controlsLayout->addItem(brightnessSlider);

    Plasma::Label *volumeLabel = new Plasma::Label(controls);
    volumeLabel->setText(i18n("Adjust Volume"));
    volumeLabel->nativeWidget()->setWordWrap(false);
    controlsLayout->addItem(volumeLabel);

    Plasma::Slider *volumeSlider = new Plasma::Slider(controls);
    volumeSlider->setRange(0, 100);
    volumeSlider->nativeWidget()->setTickInterval(20);
    volumeSlider->setValue(50);
    volumeSlider->setOrientation(Qt::Horizontal);
    controlsLayout->addItem(volumeSlider);
    brightnessLabel->nativeWidget()->setWordWrap(false);

    controls->setLayout(controlsLayout);
    item->setWidget(controls);
}

#include "mid_control.moc"
