/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/


// own
#include "buttoncontainer.h"

// Qt
#include <QGraphicsSceneResizeEvent>
#include <QGraphicsLinearLayout>

// KDE
#include <Plasma/ToolButton>
#include <KIcon>
#include <kiconloader.h>
#include <KDebug>
#include <Plasma/Animator>
#include <Plasma/Animation>



ButtonContainer::ButtonContainer(QGraphicsWidget *parent)
    : Plasma::Frame(parent)
{

    setFrameShadow(Plasma::Frame::Sunken);
    
    m_fadeAnimation = Plasma::Animator::create(Plasma::Animator::FadeAnimation);
    m_fadeAnimation->setTargetWidget(this);

    m_firstShow = true;
    
    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(Qt::Horizontal);
    layout->setContentsMargins(0, 0, 0, 0);
    
    Plasma::ToolButton *upButton = new Plasma::ToolButton(this);
    Plasma::ToolButton *downButton = new Plasma::ToolButton(this);
    Plasma::ToolButton *editButton = new Plasma::ToolButton(this);
    Plasma::ToolButton *removeButton = new Plasma::ToolButton(this);

    upButton->setIcon(KIcon("arrow-up"));
    downButton->setIcon(KIcon("arrow-down"));
    editButton->setIcon(KIcon("edit-rename"));
    removeButton->setIcon(KIcon("list-remove"));

    connect(upButton, SIGNAL(clicked()), this, SIGNAL(up()));
    connect(downButton, SIGNAL(clicked()), this, SIGNAL(down()));
    connect(editButton, SIGNAL(clicked()), this, SIGNAL(edit()));
    connect(removeButton, SIGNAL(clicked()), this, SIGNAL(remove()));
    
    layout->addItem(upButton);
    layout->addItem(downButton);
    layout->addItem(editButton);
    layout->addItem(removeButton);
    
    setLayout(layout);
    
}


ButtonContainer::~ButtonContainer()
{

    delete m_fadeAnimation;
    
}


void ButtonContainer::show()
{
    
    if (!isVisible()) {
        setOpacity(0.0);
        Plasma::Frame::show();
    }
    
    if ((opacity() == 1.0 && m_fadeAnimation->state() != Plasma::Animation::Running) ||
        (m_fadeAnimation->state() == Plasma::Animation::Running && 
        m_fadeAnimation->property("targetOpacity") == 1.0)) {
        return;
    }
    
    if (m_fadeAnimation->state() == Plasma::Animation::Running) {
        m_fadeAnimation->stop();
    }
    m_fadeAnimation->setProperty("startOpacity", opacity());
    m_fadeAnimation->setProperty("targetOpacity", 1.0);
    m_fadeAnimation->start();

}


void ButtonContainer::hide()
{

    if ((opacity() == 0.0 && m_fadeAnimation->state() != Plasma::Animation::Running) ||
        (m_fadeAnimation->state() == Plasma::Animation::Running && 
        m_fadeAnimation->property("targetOpacity") == 0.0)) {
        return;
    }
    
    if (m_fadeAnimation->state() == Plasma::Animation::Running) {
        m_fadeAnimation->stop();
    }
    m_fadeAnimation->setProperty("startOpacity", opacity());
    m_fadeAnimation->setProperty("targetOpacity", 0.0);
    m_fadeAnimation->start();
    
}


void ButtonContainer::setSortingEnabled(const bool &enabled)
{
    
    static_cast<Plasma::ToolButton*>(layout()->itemAt(0))->setEnabled(enabled);
    static_cast<Plasma::ToolButton*>(layout()->itemAt(1))->setEnabled(enabled);

}


void ButtonContainer::hideFinished()
{

    Plasma::Frame::hide();

}


QSizeF ButtonContainer::sizeHint(Qt::SizeHint which, const QSizeF &constraint) const
{

    QSizeF hint = QSizeF(0, 0);
    
    if (!layout()) {
        return hint;
    }
    
    QGraphicsLinearLayout *lay = static_cast<QGraphicsLinearLayout*>(layout());
    
    qreal left, top, right, bottom;
    getContentsMargins(&left, &top, &right, &bottom);
    
    qreal width, height;
    width = height = 0;

    for (int i = 0; i < lay->count(); i++) {
        Plasma::ToolButton *button = static_cast<Plasma::ToolButton*>(lay->itemAt(i));
        width += button->effectiveSizeHint(Qt::MinimumSize).width();
        if (!height) {
            height = button->effectiveSizeHint(Qt::MinimumSize).height();
        }
        width += lay->spacing();
    }
    
    width += left+right;
    height += top+bottom;
    
    width = qMax(width, QGraphicsWidget::sizeHint(which, constraint).width());
    
    hint.setWidth(width);
    hint.setHeight(height);
        
    return hint;
    
}


void ButtonContainer::mousePressEvent(QGraphicsSceneMouseEvent *event)
{

    event->ignore();

}
    

void ButtonContainer::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{

    event->ignore();

}


void ButtonContainer::wheelEvent(QGraphicsSceneWheelEvent *event)
{

    event->ignore();

}


#include "buttoncontainer.moc"
