/***************************************************************************
 *   Copyright (C) 2009 by Kai Dombrowe <just89@gmx.de>                    *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef HOVERWIDGET
#define HOVERWIDGET

// Qt
#include <QGraphicsWidget>
#include <QPointer>


namespace Plasma {
    class FrameSvg;
    class Animation;
}

class HoverWidget : public QGraphicsWidget
{
    Q_OBJECT


public:
    HoverWidget(QGraphicsWidget *parent);
    ~HoverWidget();

    qreal spacing() const;
    QRectF getHoverRect(QRectF targetRect, const QPointF &pos);
    
    void getMargins(qreal &left, qreal &top, qreal &right, qreal &bottom);
    void show();
    void hide();
    
    
public slots:
    void setTarget(QGraphicsWidget *widget);
    void move(const QPointF &newPos);

    
private slots:
    void syncBorders(); 
    void hideFinished();
    

private:
    Plasma::FrameSvg *m_svg;
    QPointer<QGraphicsWidget> m_target;
    Plasma::Animation *m_fadeAnimation;
    Plasma::Animation *m_moveAnimation;
    

protected:
    void paint(QPainter *painter, const QStyleOptionGraphicsItem *options, QWidget *widget = 0);
        
    void mousePressEvent(QGraphicsSceneMouseEvent *event);
    void keyPressEvent(QKeyEvent *event);
    void wheelEvent(QGraphicsSceneWheelEvent *event);
    void mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event);
    void hoverMoveEvent(QGraphicsSceneHoverEvent *event);
    void hoverLeaveEvent(QGraphicsSceneHoverEvent *event);
    void resizeEvent(QGraphicsSceneResizeEvent *event);
    void mouseReleaseEvent(QGraphicsSceneMouseEvent *event);

    bool eventFilter(QObject *watched, QEvent *event);

    
};


#endif

