#include "outputframe.h"
#include "plasma/widgets/label.h"
#include "plasma/widgets/combobox.h"
#include "plasma/widgets/radiobutton.h"
#include <QString>
#include <QGraphicsGridLayout>
#include <QGraphicsLinearLayout>
#include <QProcess>
#include <QButtonGroup>
#include <QRadioButton>
#include <KComboBox>
#include <kephal/configurations.h>
#include <kephal/kephal.h>
#include <KDebug>

OutputFrame::OutputFrame( Kephal::Output* output, QGraphicsWidget *parent ) :
    Plasma::Frame(parent), m_output(output) {
	setSizePolicy(QSizePolicy::Expanding, QSizePolicy::Expanding);

	m_layout = new QGraphicsGridLayout(this);
	m_layout->setSpacing(1);
	setMinimumSize(350,70);
	//setMaximumSize(1000,1000);
	//setMaximumHeight( 120 );
	setLayout( m_layout );

	//vendor_label = new Plasma::Label( this );
	//    m_layout->addItem( vendor_label, 0, 0 );

	//output_icon = new Plasma::IconWidget( QIcon("devices/video-display"), "", this );
	/*output_icon = new Plasma::IconWidget( KIcon("preferences-desktop-display-randr"), output->id(), this );
	output_icon -> setSizePolicy(QSizePolicy::Ignored, QSizePolicy::Ignored);
	output_icon -> setMinimumSize(70,70);
	m_layout->addItem( output_icon, 0, 0, 4, 1 );*/

	    resolution_label = new Plasma::Label( this );
	    resolution_label -> setText( i18n( "Resolution" ) + ":" );
	    m_layout->addItem( resolution_label, 1, 0 );

	    resolution_selector = new Plasma::ComboBox( this );
	    m_layout -> addItem( resolution_selector, 1, 1, Qt::AlignTop );
	    resolution_selector -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	    resolution_selector -> nativeWidget() -> setSizeAdjustPolicy( QComboBox::AdjustToContentsOnFirstShow );

	    resolution_selector -> addItem( i18n( "Deactivated" ) );

	    refresh_label = new Plasma::Label( this );
	    refresh_label -> setText( i18n( "Refresh Rate" ) + ":" );
	    m_layout->addItem( refresh_label, 1, 3 );

	    refresh_selector = new Plasma::ComboBox( this );
	    m_layout -> addItem( refresh_selector, 1, 4, Qt::AlignTop );
	    refresh_selector -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum);
	    refresh_selector -> nativeWidget() -> setSizeAdjustPolicy( QComboBox::AdjustToContentsOnFirstShow );


	    orientation_label = new Plasma::Label( this );
	    orientation_label -> setText( i18n( "Orientation" ) + ":" );
	    m_layout->addItem( orientation_label, 2, 0 );

	    m_buttonline = new QGraphicsLinearLayout();
	    m_buttonline->setOrientation( Qt::Horizontal );
	    m_buttonline->setSpacing(1);
	    m_layout->addItem( m_buttonline, 2, 1, 1, 4, Qt::AlignVCenter);

	    m_buttongroup = new QButtonGroup(this);

	    button_normal = new Plasma::RadioButton( this );
	    button_normal -> setText(QString("0")+QChar(176));
	    //button_normal -> setMinimumSize(5,5);
	    m_buttongroup -> addButton(button_normal->nativeWidget());
	    //button_normal -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, QSizePolicy::DefaultType);
	    m_buttonline->addItem( button_normal );

	    button_right = new Plasma::RadioButton( this );
	    button_right -> setText(QString("90")+QChar(176));
	    //button_right -> setMinimumSize(5,5);
	    m_buttongroup -> addButton(button_right->nativeWidget());
	    //button_right -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, QSizePolicy::DefaultType);
	    m_buttonline->addItem( button_right );

	    button_inverted = new Plasma::RadioButton( this );
	    button_inverted -> setText(QString("180")+QChar(176));
	    //button_inverted -> setMinimumSize(5,5);
	    m_buttongroup -> addButton(button_inverted->nativeWidget());
	    //button_inverted -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, QSizePolicy::DefaultType);
	    m_buttonline->addItem( button_inverted );

	    button_left = new Plasma::RadioButton( this );
	    button_left -> setText(QString("270")+QChar(176));
	    m_buttongroup -> addButton(button_left->nativeWidget());
	    //button_left -> setSizePolicy(QSizePolicy::Maximum, QSizePolicy::Maximum, QSizePolicy::DefaultType);
	    m_buttonline->addItem( button_left );

	    init();

	    // connect the signals AFTER having added text entrys to boxes

	    connect( resolution_selector -> nativeWidget(), SIGNAL( currentIndexChanged( int ) ), this, SLOT( setChanged() ));
	    connect( refresh_selector -> nativeWidget(), SIGNAL( currentIndexChanged( int ) ), this, SLOT( setChanged() ));
	    connect( m_buttongroup, SIGNAL( buttonClicked( int ) ), this, SLOT( setChanged() ));

	    connect( Kephal::Outputs::self(), SIGNAL( outputResized( Kephal::Output*, QSize, QSize ) ), this, SLOT( init() ));
	    connect( Kephal::Outputs::self(), SIGNAL( outputActivated( Kephal::Output* ) ), this, SLOT( init() ));
	    connect( Kephal::Outputs::self(), SIGNAL( outputDeactivated( Kephal::Output* ) ), this, SLOT( selectResolution() ));
	    connect( Kephal::Outputs::self(), SIGNAL( outputRotated( Kephal::Output*, Kephal::Rotation, Kephal::Rotation ) ), this, SLOT( init() ));
	    connect( Kephal::Outputs::self(), SIGNAL( outputRateChanged( Kephal::Output*, float, float ) ), this, SLOT( init() ));

}

OutputFrame::~OutputFrame()
{
}

Kephal::Output* OutputFrame::output()
{
    return m_output;
}

void OutputFrame::init() {
	//vendor_label -> setText( m_output -> vendor() );

	resolution_selector -> nativeWidget() -> clear();

	resolution_selector -> addItem( i18n( "Deactivated" ) );
	foreach(QSize resolution, m_output->availableSizes() ){
	    if ( resolution.width() > 0 ) {
		resolution_selector -> nativeWidget() -> addItem( QVariant( resolution.width() ).toString()  + QString("x") + QVariant( resolution.height() ).toString(), resolution );
	    }
	}

	refresh_selector -> nativeWidget() -> clear();
	foreach(float rate, m_output->availableRates() ){
		refresh_selector -> nativeWidget() -> addItem( QVariant( QVariant( rate ).toInt() ).toString() + " Hz", rate );
	}

	selectResolution();
	selectRate();
	selectOrientation();
}

void OutputFrame::selectResolution() {
	if( m_output->isActivated() ){
	    int index = resolution_selector -> nativeWidget() -> findData ( m_output -> size() );
	    resolution_selector -> nativeWidget() -> setCurrentIndex( index );
	}
	else{
	    resolution_selector -> nativeWidget() -> setCurrentIndex( 0 );
	}
}

void OutputFrame::selectRate() {
	int index = refresh_selector -> nativeWidget() -> findData ( m_output -> rate() );
	refresh_selector -> nativeWidget() -> setCurrentIndex( index );
}

void OutputFrame::selectOrientation() {
	Kephal::Rotation orientation = m_output -> rotation();
	switch(orientation){
	    case Kephal::RotateNormal:
		button_normal->setChecked(true);
		break;
	    case Kephal::RotateRight:
		button_right->setChecked(true);
		break;
	    case Kephal::RotateInverted:
		button_normal->setChecked(true);
		break;
	    case Kephal::RotateLeft:
		button_left->setChecked(true);
		break;
	}
}


void OutputFrame::setChanged() {
	m_changed=true;
}

void OutputFrame::setUnChanged() {
	m_changed=false;
}

bool OutputFrame::changed() {
	return m_changed;
}

void OutputFrame::acceptConfiguration()
{   
	if(!changed())
	{
	    return;
	}

	setUnChanged() ;

	int index = resolution_selector -> nativeWidget() -> currentIndex();
	QString command = QString("xrandr --output " + m_output->id());

	// handle resolution

	if ( index == 0 ){
	    QProcess::execute ( command + " --off");
	    return;
	}
	else {
	    QSize size = resolution_selector -> nativeWidget() -> itemData( index ).toSize();
	    command += " --mode " + QVariant(size.width()).toString() +"x"+ QVariant(size.height()).toString();
	}
	kDebug() << command;
	QProcess::execute ( command );

	// handle rates
	command = QString("xrandr --output " + m_output->id());

	index = refresh_selector -> nativeWidget() -> currentIndex();
	QString rate = refresh_selector -> nativeWidget() -> itemData( index ).toString();
	command += " --rate " + rate;
	kDebug() << command;
	QProcess::execute ( command );

	// handle orientation
	command = QString("xrandr --output " + m_output->id());
	QString orientation = m_buttongroup->checkedButton()->text();
	orientation.chop(1);

	int degrees = QVariant(orientation).toInt();
	
	switch(degrees)
	{
		case 0:
		    command += " --rotate normal";
		    break;
		case 90:
		    command += " --rotate right";
		    break;
		case 180:
		    command += " --rotate inverted";
		    break;
		case 270:
		    command += " --rotate left";
		    break;
	}
	kDebug() << command;
	QProcess::execute ( command );
}


/*
void OutputFrame::resize( int index ) {
	//vendor_label -> setText( resolution_selector -> nativeWidget() -> itemValue( index ) );
	//m_output -> resize( resolution_selector -> nativeWidget() -> itemData( index ).toSize() );
	//m_output -> resize( resolution_selector -> nativeWidget() -> itemData( index ).toSize() );
	//Kephal::Configurations::self()->resize ( m_output, resolution_selector -> nativeWidget() -> itemData( index ).toSize() );
	//Kephal::Configurations::self() -> resize ( m_output, QSize(1024,768) );
	//m_output -> resize( QSize(1024,768) );
	if(quiet()){
	    return;
	}

	if ( index == 0 ){
	    QProcess::execute ( "xrandr --output " + m_output->id() + " --off");
	}
	else {
	    QSize size = resolution_selector -> nativeWidget() -> itemData( index ).toSize();
	    QProcess::execute ( "xrandr --output " + m_output->id() + " --mode " + QVariant(size.width()).toString() +"x"+ QVariant(size.height()).toString() );
	}
}

void OutputFrame::changeRate( int index ) {
	if(quiet()){
	    return;
	}
	QString rate = refresh_selector -> nativeWidget() -> itemData( index ).toString();
	QProcess::execute ( "xrandr --output " + m_output->id() + " --rate " + rate );
	//Kephal::Configurations::self() -> resize ( m_output, QSize(1024,768) );
}

void OutputFrame::setQuiet(bool quiet)
{
	beQuiet = quiet;
}

bool OutputFrame::quiet()
{
	return beQuiet;
}
*/
