/***************************************************************************
 *   Copyright 2008 by Beat Wolf <asraniel@fryx.ch>                        *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "serverapplet.h"


#include <QGraphicsGridLayout>

#include <KIcon>
#include <KRun>
#include <QVariant>

#include <QAction>
#include <QList>
#include <QActionGroup>
#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/widgets/iconwidget.h>

#include <QPushButton>
#include <QLabel>
#include <QGraphicsProxyWidget>

ServerApplet::ServerApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
    m_base_services = new QList<struct service>();
    struct service fish = {"Fish", "fish", "folder-remote", SERVICE};
    struct service ftp = {"Ftp", "ftp", "face-sad", SERVICE};
    struct service http = {"Http", "http", "folder-html", SERVICE};
    struct service smb = {"Samba", "smb", "face-sad", SERVICE};
    struct service ssh = {"Ssh", "--ssh--", "utilities-terminal", SHELL};

    m_base_services->append(fish);
    m_base_services->append(ftp);
    m_base_services->append(http);
    m_base_services->append(smb);
    m_base_services->append(ssh);

    m_services = new QActionGroup(this);
    connect(m_services, SIGNAL(triggered( QAction *)), this, SLOT(openService(QAction *)));

    m_icon = new KIcon("network-server");
    m_name = i18n("No Server");

    setHasConfigurationInterface(true);
    setAspectRatioMode(Plasma::ConstrainedSquare);
    setBackgroundHints(NoBackground);

    //layout code
    resize(200,200);
}


ServerApplet::~ServerApplet()
{
    delete m_services;
    delete m_icon;
}

void ServerApplet::init()
{
    KConfigGroup config = this->config();
    m_name = config.readEntry("name", m_name);
    m_address = config.readEntry("address", QString());
    m_username = config.readEntry("username", QString());

    QGraphicsGridLayout *layout = new QGraphicsGridLayout;
    Plasma::IconWidget *g_icon = new Plasma::IconWidget(*m_icon, "", this);

    //FIXME: use Plasma::Label
    m_name_label = new QLabel(m_name);
    m_name_label->setAlignment(Qt::AlignHCenter);
    QGraphicsProxyWidget *g_name = new QGraphicsProxyWidget;
    g_name->setWidget(m_name_label);

    //FIXME: this should just be a layout used to put Plasma::PushButtons in
    m_actions = new QWidget();
    layout_actions = new QVBoxLayout();
    m_actions->setLayout(layout_actions);
    QGraphicsProxyWidget *g_actions = new QGraphicsProxyWidget;
    g_actions->setWidget(m_actions);

    layout->addItem(g_icon,0,0);
    layout->addItem(g_name,1,0);
    layout->addItem(g_actions,0,1,2,1);

    layout->setRowStretchFactor(0,1);
    layout->setColumnStretchFactor(0,1);

    setLayout(layout);

    constructServices();

    if (m_address.isEmpty()) {
        setConfigurationRequired(true);
    }
}

void ServerApplet::openService(QAction * action){
    if(action->data().toString().compare("--ssh--") == 0){
        QString command = "konsole -e ssh -l " + m_username  + " " + m_address;
        qDebug() << command;
        KRun::runCommand(command,0);
    } else {
        new KRun(action->data().toString()+"://" + m_address, 0);
    }
}

//FIXME: port to createConigurationInterface
void ServerApplet::showConfigurationInterface()
{
    KConfigGroup config = this->config();
    Configuration conf(config, m_base_services);

    conf.exec();

    m_name = config.readEntry("name", i18n("No Server"));
    m_address = config.readEntry("address", QString());
    setConfigurationRequired(m_address.isEmpty());
    constructServices();
    m_name_label->setText(m_name);
}

void ServerApplet::constructServices()
{
    QList<QAction *> actions = m_services->actions();
    for(int i = 0; i < actions.size(); i++){
        m_services->removeAction(actions.at(i));
        delete actions.at(i);
    }

    delete m_actions->layout();
    while(m_actions->children().size() > 0){
        delete m_actions->children().at(0);
    }

    layout_actions = new QVBoxLayout();
    m_actions->setLayout(layout_actions);

    QStringList list = config().readEntry("services", QStringList());

    for(int i = 0; i < list.size() ; i++){
        //TODO: find a way to get pretty icons
        QString icon;
        QString name;
        for(int loop = 0; loop < m_base_services->size();loop++){
            if(m_base_services->at(loop).protocol.compare(list.at(i)) == 0){
                icon = m_base_services->at(loop).icon;
                name = m_base_services->at(loop).name;
            }
        }

        KIcon l_icon(icon);
        QAction *tempAction = new QAction(l_icon,list.at(i),this);
        tempAction->setData(QVariant( list.at(i) ));
        m_services->addAction(tempAction);

        QPushButton *but = new QPushButton(l_icon, name, m_actions);
        ((QVBoxLayout *)(m_actions->layout()))->addWidget(but);
        connect(but, SIGNAL(released()), tempAction, SLOT(trigger()));
    }
}

#include "serverapplet.moc"
