#ifndef __SVGLAYOUTITEM_H__
#define __SVGLAYOUTITEM_H__

#include <Plasma/Svg>
#include <QGraphicsItem>
#include <QGraphicsLayoutItem>

namespace Plasma {

/** Provides theme aware SVG rending in a QGraphicsLayoutItem
 * subclass. This way you can use QGraphicsLayout and it subclasses
 * to layout you SVG items.
 * You can also use this class in derived classes
 * to render a background and override paint to draw your own contents.
 */
class SvgLayoutItem : public QGraphicsItem, public QGraphicsLayoutItem
{
public:
    SvgLayoutItem(QGraphicsItem *parent = 0);
    virtual ~SvgLayoutItem();

    // reimplemented from QGraphicsLayoutItem
    virtual QSizeF sizeHint(Qt::SizeHint which, const QSizeF &constraint = QSizeF()) const;
    void setGeometry(const QRectF &rect);
    // reimplemtned from QGraphicsItem
    virtual QRectF boundingRect() const;
    virtual void paint(QPainter*, const QStyleOptionGraphicsItem*, QWidget*);
    
    void setImagePath(const QString &path);

private:
    Svg m_svg;
    QRectF m_boundingRect;
    qreal m_dblTop, m_dblBottom, m_dblLeft, m_dblRight;
};

}

#endif


