# -*- coding: utf-8 -*- 
#
#   Copyright (C) 2009 Andrew Stromme <astromme@chatonka.com>
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License version 2,
#   or (at your option) any later version, as published by the Free
#   Software Foundation
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details
#   You should have received a copy of the GNU General Public
#   License along with this program; if not, write to the
#   Free Software Foundation, Inc.,
#   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
#

from PyQt4.QtCore import *
from PyQt4.QtGui import *
from PyKDE4.kdecore import i18n
from PyKDE4.kdeui import *
from PyKDE4.plasma import Plasma
from PyKDE4 import plasmascript
import dbus
 
class SuspendPlasmoid(plasmascript.Applet):
    def __init__(self,parent,args=None):
        plasmascript.Applet.__init__(self,parent)
 
    def init(self):
        self.setHasConfigurationInterface(False)
	self.setAspectRatioMode(Plasma.IgnoreAspectRatio)

	if (self.formFactor() == Plasma.Planar):
	  self.planar = True
	  self.horizontal = False
	elif (self.formFactor() == Plasma.Horizontal):
	  self.planar = False
	  self.horizontal = True
	elif (self.formFactor() == Plasma.Vertical):
	  self.planar = False
	  self.horizontal = False
	
	self.sessionBus = dbus.SessionBus()
	self.powermanagement = self.sessionBus.get_object('org.freedesktop.PowerManagement',
                      '/org/freedesktop/PowerManagement')
	self.powerdevil = self.sessionBus.get_object('org.freedesktop.PowerManagement',
                      '/modules/powerdevil')
 
        self.theme = Plasma.Svg(self)
        self.theme.setImagePath("widgets/background")
        self.setBackgroundHints(Plasma.Applet.DefaultBackground)
 
	if (not self.planar) and (self.horizontal):
	  self.layout = QGraphicsLinearLayout(Qt.Horizontal, self.applet)
	else:
	  self.layout = QGraphicsLinearLayout(Qt.Vertical, self.applet)

	if self.planar:
	  self.label = Plasma.Label(self.applet)
	  self.label.setText(i18n("Power Options:"))
	  self.layout.addItem(self.label)

        screenoff = Plasma.PushButton(self.applet)
        screenoff.nativeWidget().setIcon(KIcon("preferences-desktop-display"))
        if self.planar:
          screenoff.setText(i18n("Turn Display Off"))
        else:
          screenoff.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
          screenoff.resize(screenoff.size().height(), screenoff.size().height())
        self.layout.addItem(screenoff)
        QObject.connect(screenoff, SIGNAL("clicked()"), self.displayOff)

	if (self.powermanagement.CanSuspend(dbus_interface='org.freedesktop.PowerManagement') == dbus.Boolean(True)):	 
	  suspendbutton = Plasma.PushButton(self.applet)
	  suspendbutton.nativeWidget().setIcon(KIcon("system-suspend"))
	  if self.planar:
	    suspendbutton.setText(i18n("Suspend"))
	  else:
	    suspendbutton.resize(suspendbutton.size().height(), suspendbutton.size().height())
	    suspendbutton.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
	  self.layout.addItem(suspendbutton)
	  QObject.connect(suspendbutton, SIGNAL("clicked()"), self.suspend)

	if (self.powermanagement.CanHibernate(dbus_interface='org.freedesktop.PowerManagement') == dbus.Boolean(True)):
	  hibernate = Plasma.PushButton(self.applet)
	  hibernate.nativeWidget().setIcon(KIcon("system-suspend-hibernate"))
	  if self.planar:
	    hibernate.setText(i18n("Hibernate"))
	  else:
	    hibernate.resize(hibernate.size().height(), hibernate.size().height())
	    hibernate.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
	  self.layout.addItem(hibernate)
	  QObject.connect(hibernate, SIGNAL("clicked()"), self.hibernate)

        self.setLayout(self.layout)
	if self.planar:
	  self.resize(200,125)
	elif self.horizontal:
	  self.resize(24, 72)
	else:
	  self.resize(72, 24)
	  

    def displayOff(self):
	self.powerdevil.turnOffScreen(dbus_interface='org.kde.PowerDevil')

    def hibernate(self):
	self.powermanagement.Hibernate(dbus_interface='org.freedesktop.PowerManagement')

    def suspend(self):
	self.powermanagement.Suspend(dbus_interface='org.freedesktop.PowerManagement')

 
def CreateApplet(parent):
    return SuspendPlasmoid(parent)