/*
 *   Copyright (C) 2009 Aaron Seigo <aseigo@kde.org>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License version 2 as
 *   published by the Free Software Foundation
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef TIMELINEAPPLET_H
#define TIMELINEAPPLET_H

#include <QDateTime>
#include <QTimer>

#include <Plasma/Applet>
#include <Plasma/DataEngine>

class TimeLineApplet : public Plasma::Applet
{
    Q_OBJECT

public:
    enum Zoom { HourZoom = 10, DayZoom = 30, WeekZoom = 120, MonthZoom = 1440 };

    TimeLineApplet(QObject *parent, const QVariantList &args);
    ~TimeLineApplet();

    void init();

public Q_SLOTS:
    void dataUpdated(const QString &name, const Plasma::DataEngine::Data &data);

protected:
    void createConfigurationInterface(KConfigDialog *parent);
    void paintInterface(QPainter *painter,
                        const QStyleOptionGraphicsItem *option,
                        const QRect &contentsRect);

private Q_SLOTS:
    void configAccepted();
    void moveTimeLine();

private:
    QTimer m_moveTimer;
    QDateTime m_now;
    QDateTime m_then;
    Zoom m_zoomLevel;
    int m_stepSize;
    int m_moveOffset;
};

#endif
