/*
# * Copyright(c) 2008 Matthew Adams <roguestar191 at comcast.net>
# *
# * This program is free software; you can redistribute it and/or
# * modify it under the terms of the GNU General Public License as
# * published by the Free Software Foundation; either version 2 of
# * the License, or(at your option) any later version.
# *
# * This program is distributed in the hope that it will be useful,
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with this program.  If not, see <http//www.gnu.org/licenses/>.
# */
#ifndef Ui_PlasmaTools
#define Ui_PlasmaTools

#include <QtCore/QVariant>
#include <QtGui/QAction>
#include <QtGui/QApplication>
#include <QtGui/QButtonGroup>
#include <QtGui/QComboBox>
#include <QtGui/QDialog>
#include <QtGui/QFrame>
#include <QtGui/QGridLayout>
#include <QtGui/QLabel>
#include <QtGui/QListWidget>
#include <QtGui/QProgressBar>
#include <QtGui/QScrollArea>
#include <QtGui/QSpacerItem>
#include <QtGui/QTabWidget>
#include <QtGui/QVBoxLayout>
#include <QtGui/QWidget>
#include <QtWebKit/QWebView>
#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsWidget>
#include <QGraphicsSceneMouseEvent>
#include <QStandardItemModel>
#include <QModelIndex>
#include <QAction>
#include <QTimer>
#include <QTreeView>
#include <QWebPage>
#include <QWebFrame>
#include <QWebHistory>
#include <QListWidget>
#include <QScrollArea>

#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QTimer>
#include "karrowbutton.h"
#include "kpushbutton.h"
#include "kurlrequester.h"
#include <kglobalsettings.h>
#include <khistorycombobox.h>
#include <KIcon>
#include <KCompletion>
#include <KBookmarkManager>
#include <KIconLoader>
#include <KUrlPixmapProvider>
#include <KUriFilter>
#include <KMessageBox>
#include <KConfigDialog>
#include <KHistoryComboBox>


#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/widgets/iconwidget.h>
#include <plasma/widgets/pushbutton.h>
#include <plasma/framesvg.h>
#include <plasma/widgets/label.h>
#include <plasma/applet.h>
#include <plasma/widgets/webview.h>
#include <plasma/widgets/slider.h>
#include <plasma/widgets/combobox.h>
#include <Plasma/IconWidget>
#include <Plasma/LineEdit>
#include <Plasma/Meter>
#include <Plasma/WebView>
#include <Plasma/TreeView>
#include <Plasma/Slider>
#include "../source/tabbar.h"
#include "../source/webview.h"
#include <vector>

namespace welcome_plasmoid {
	class Ui_WelcomePlasmoid;
	class ScrollableListWidget : public QGraphicsProxyWidget {
		public:
		ScrollableListWidget(QGraphicsItem *top);

                QScrollArea *ScrollArea;
                QWidget *ScrollWidget;
                QListWidget *ListWidget;
		QGraphicsLinearLayout *Layout;
		QVBoxLayout *ScrollLayout;
	};
	class aPid {
		public:
			class aFp { 
				public: 
				FILE*fp; 
				int count; 
				aPid *top;
				aFp(FILE*_fp, aPid* _top);
				aFp(const aFp& old);
				~aFp();
			};
			void addFile(FILE*fp);
			void remFile(FILE*fp);
			static std::vector<aFp> files;
			bool alive;
			FILE *fp;
			QString buffer;
			aPid(FILE* _fp);
			aPid(const aPid &old);
			~aPid();
			ssize_t read();
	};
	QString runCommand(QString cmd);

	class Dialog_Tools { //Inherited by all the "tabs" so they can "communicate" easily with their parent.
		public:
			Dialog_Tools();
			virtual ~Dialog_Tools();

			void clearUrl();

			void invalidateLayouts();
	};
	class Url_Proxy : public QGraphicsProxyWidget {
		Q_OBJECT
		public:
			KHistoryComboBox *url;
			Ui_WelcomePlasmoid* top;
		public: signals:
			void textChanged(QString);
			void returnPressed(QString);
		public:
			Url_Proxy(Ui_WelcomePlasmoid* parent);
	};

	class TransparentFrame : public QGraphicsWidget {
		Q_OBJECT
		private:
		Ui_WelcomePlasmoid *top;
		public:
			TransparentFrame(Ui_WelcomePlasmoid* parent);
	};
	
	class ResizeFrame : public TransparentFrame {
		Q_OBJECT
		QGraphicsGridLayout *gridlayout;
		QGraphicsLinearLayout *linearlayout;
		Ui_WelcomePlasmoid *top;
		public:
			ResizeFrame(Ui_WelcomePlasmoid* par=NULL);
			void setLayout(QGraphicsGridLayout* l);
			void setLayout(QGraphicsLinearLayout* l);
		protected:
			virtual void resizeEvent(QGraphicsSceneResizeEvent*e);
			virtual void updateGeometry();
	};
	
	class WebButton : public Plasma::PushButton {
		public:
			WebButton(Ui_WelcomePlasmoid* parent=NULL);
			void setLabelText(QString&);
	};
	class GloweyLabel;
	class MyQtLabel : public QLabel {
		Q_OBJECT
		public:
			QString currentLink;
			bool InsideWidget;
			int brightness;
			QTimer *mytimer;
			Ui_WelcomePlasmoid* top;
			GloweyLabel* par;
			bool waveup;
			float wave;
			float wavetop;
			Plasma::FrameSvg *backgroundImage;
			QImage mypixmap;
			bool hasPixmap;
			bool animating;
			QPoint point;
			QPainter *painter;
		public slots:
			void linkHovered(QString);
			void animate();
		public: signals:
			void clickedOn(QString);
		public:
			MyQtLabel(int, QWidget *par);
			MyQtLabel(GloweyLabel* par=NULL, Ui_WelcomePlasmoid* _top=NULL, QWidget *_par=NULL);
			~MyQtLabel(){};
			void setup(GloweyLabel* par=NULL, Ui_WelcomePlasmoid* _top=NULL);
			virtual void mousePressEvent(QMouseEvent*e);
			void generatePalette();
			void resizeEvent(QResizeEvent*e);
			void paintEvent(QPaintEvent*e);
			void leaveEvent(QEvent*e);
			void startAnimate();
	};
	
	class GloweyLabel : public Plasma::Label {
		Q_OBJECT
		public:
			MyQtLabel *labelWidget;
			QScrollArea *scrollArea;
			bool isSetup;
			QString iconpath;
			QString filepath;
			QString description;
			QString name;
			Plasma::Svg *icon;
			QString launcher;
			bool scroll;
		public slots:
			void linkHovered();
			void linkClicked();
			void setName(QString);
			void setDescription(QString);
			void setIcon(QString);
			void setLauncher(QString);
		public: signals:
			void linkActivated(QString);
		public:
			GloweyLabel();
			GloweyLabel(Ui_WelcomePlasmoid* top, bool setMax=true);
			~GloweyLabel(){};
			void setup(Ui_WelcomePlasmoid* top, bool setMax=true, bool useScroll=false);
			void setMaximumSize(int x, int y);
			void setMinimumSize(int x, int y);
			void setLabelText(QString);
			void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget=0);			
	};
	class GloweyIcon : public Plasma::IconWidget {
		Q_OBJECT
			bool InsideWidget;
			int brightness;
			QTimer *mytimer;
			Ui_WelcomePlasmoid* top;
			GloweyLabel* par;
			bool waveup;
			QString link;
			float wave;
			float wavetop;
			Plasma::FrameSvg *backgroundImage;
			QImage mypixmap;
			bool hasPixmap;
			bool animating;
			QPoint point;
			bool activated;
			bool postactivated;
			QTimer *t;
		public:	signals:
				void linkActivated(QString);
		public:
		
			GloweyIcon(Ui_WelcomePlasmoid* _top);
			void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget=0);
			void hoverEnterEvent(QGraphicsSceneHoverEvent*);
			void hoverLeaveEvent(QGraphicsSceneHoverEvent*);
			void resizeEvent( QGraphicsSceneResizeEvent * event );
		public slots:
			void postActivate();
			void animate();
			void setLink(QString);
		private slots:
			void linkClicked();
	};
}

#endif

