/*
    Copyright 2007 Robert Knight <robertknight@gmail.com>

    This library is free software; you can redistribute it and/or
    modify it under the terms of the GNU Library General Public
    License as published by the Free Software Foundation; either
    version 2 of the License, or (at your option) any later version.

    This library is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Library General Public License for more details.

    You should have received a copy of the GNU Library General Public License
    along with this library; see the file COPYING.LIB.  If not, write to
    the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
    Boston, MA 02110-1301, USA.
*/

#include "windowsapplet.h"

// Qt
#include <QLabel>
#include <QComboBox>
#include <QGridLayout>
#include <QGraphicsView>
#include <QMetaObject>
#include <QMetaEnum>
#include <QPointer>
#include <QGraphicsLinearLayout>

// KDE
#include <KIcon>
#include <KConfigDialog>
#include <KMenu>
#include <KProcess>

// Plasma
#include <plasma/containment.h>

#include <windows.h>

WindowsLauncherApplet::WindowsLauncherApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent,args)
{
    setBackgroundHints(NoBackground);

    resize(IconSize(KIconLoader::Desktop) * 2, IconSize(KIconLoader::Desktop) * 2);

    icon = new Plasma::IconWidget(QString(), this);
    icon->setFlag(ItemIsMovable, false);
    connect(icon, SIGNAL(pressed(bool)), this, SLOT(toggleMenu(bool)));
}

WindowsLauncherApplet::~WindowsLauncherApplet()
{
}

void WindowsLauncherApplet::init()
{
    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
    layout->setContentsMargins(0, 0, 0, 0);
    layout->setSpacing(0);

    layout->addItem(icon);

    icon->setIcon(KIcon("start-here-kde"));

    setAspectRatioMode(Plasma::ConstrainedSquare);

    constraintsEvent(Plasma::ImmutableConstraint);
}

void WindowsLauncherApplet::constraintsEvent(Plasma::Constraints constraints)
{
    setBackgroundHints(NoBackground);
    if (constraints & Plasma::FormFactorConstraint) {
        if (formFactor() == Plasma::Planar ||
            formFactor() == Plasma::MediaCenter) {
            //FIXME set correct minimum size
            //setMinimumContentSize(d->icon->sizeFromIconSize(IconSize(KIconLoader::Desktop)));
        } else {
            //setMinimumContentSize(d->icon->sizeFromIconSize(IconSize(KIconLoader::Small)));
        }
    }
}

void WindowsLauncherApplet::toggleMenu(bool pressed)
{
    if (pressed) {
            HWND hWndStartMenu = ::FindWindow("DV2ControlHost",NULL);
            if (!IsWindowVisible(hWndStartMenu)) {
                RECT rc;
                GetWindowRect(hWndStartMenu, &rc);
                QPoint p = popupPosition(QSize(rc.right-rc.left, rc.bottom-rc.top));
                SetWindowPos(hWndStartMenu,NULL,p.x(),p.y(),0,0,SWP_NOZORDER|SWP_NOSIZE);
                
                //temporary disable the dropshadow because it remains visible after hiding the menu
                int res;
                SystemParametersInfo(0x1024, 0, &res, 0); //SPI_GETDROPSHADOW
                if(res) {
                    SystemParametersInfo(0x1025, 0 , 0, 0); //SPI_SETDROPSHADOW
                }
                ShowWindow(hWndStartMenu, SW_SHOW);
                if(res) {
                    SystemParametersInfo(0x1025, 0 , &res, 0); //SPI_SETDROPSHADOW
                }
            } else {
                ShowWindow(hWndStartMenu, SW_HIDE);
            }
    }
}

#include "windowsapplet.moc"
