#include "titleapplet.h"
#include <QPainter>
#include <QFontMetrics>
#include <QSizeF>
#include <QDebug> 

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <kwindowinfo.h>
#include <kwindowsystem.h> 
#include <netwm.h>
#include <QHBoxLayout>
#include <QGraphicsProxyWidget>
#include <QGraphicsLinearLayout>
#include <QX11Info>
#include <QApplication>
#include <QLinearGradient>
#include <QLabel>

using namespace Plasma;

TitleApplet::TitleApplet(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args)
{
}


TitleApplet::~TitleApplet()
{
    if (hasFailedToLaunch()) {
        // Do some cleanup here
    } else {
        // Save settings
    }
}
 
void TitleApplet::init()
{
 
    // A small demonstration of the setFailedToLaunch function
    //if (m_icon.isNull()) {
        //setFailedToLaunch(true, "No world to say hello");
    //}

    QGraphicsLinearLayout *layout = new QGraphicsLinearLayout(this);
    layout->setContentsMargins(0,0,0,0);
    layout->setSpacing(0);

    m_windowicon = new IconWidget(KIcon("kde"),"",this);
    m_title = new Label(this);
    m_closebutton = new IconWidget(KIcon("application-exit"),"",this);

    setSizePolicy(QSizePolicy::Expanding,QSizePolicy::Expanding);
    

    layout->addItem(m_windowicon);
    layout->addItem(m_title);
    //layout->addStretch();
    layout->addItem(m_closebutton);
    layout->setStretchFactor(m_windowicon,0);
    layout->setStretchFactor(m_title,1);
    layout->setStretchFactor(m_closebutton,0);
  
    QObject::connect(m_closebutton,SIGNAL(clicked()),this,SLOT(closeClicked()));

    onActiveWindowChanged(KWindowSystem::activeWindow());
    QObject::connect(KWindowSystem::self(),SIGNAL(activeWindowChanged(WId)),this,SLOT(onActiveWindowChanged(WId)));
} 
 
void TitleApplet::paintInterface(QPainter *p,
        const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
    QRectF r = boundingRect();
    
    p->save();

    //If the desktop is not being shown, draw a title bar looking background
    if(m_title->text() != "Desktop")
    {
      p->setPen(Qt::white);
      QLinearGradient g;
      g.setColorAt(0,QApplication::palette().color(QPalette::Light));
      g.setColorAt(1,QApplication::palette().color(QPalette::Window));
      g.setSpread(QGradient::RepeatSpread);
      g.setStart(0,0);
      g.setFinalStop(0,r.height());
      p->setBrush(QBrush(g));
      p->drawRoundedRect(r,5.0,10.0);
    }

    p->restore();
    
}

void TitleApplet::onActiveWindowChanged(WId id)
{
  KWindowInfo inf = KWindowSystem::windowInfo(id,NET::WMName);
  m_title->setText(inf.name());
  if(inf.name() == "") 
  {
    m_title->setText("Desktop");
    m_windowicon->setIcon(KIcon("kde"));

    QColor c=QColor(255,255,255);    //palette().color(QPalette::WindowText);
    QPalette p = m_title->nativeWidget()->palette();
    p.setColor(QPalette::WindowText,c);
    m_title->nativeWidget()->setPalette(p);

    m_closebutton->hide();
  }
  else 
  {
    m_windowicon->setIcon(QIcon(KWindowSystem::icon(id)));
    QColor c=QApplication::palette().color(QPalette::WindowText);
    QPalette p = m_title->nativeWidget()->palette();
    p.setColor(QPalette::WindowText,c);
    m_title->nativeWidget()->setPalette(p);
    m_closebutton->show();
  }
  update();
} 

void TitleApplet::closeClicked()
{
  WId id = KWindowSystem::activeWindow();
  NETRootInfo ri( QX11Info::display(), NET::CloseWindow );
  ri.closeWindowRequest(id);
}

#include "titleapplet.moc"
