/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   Copyright (C) 2008 by Dennis Nienhüser <earthwings@gentoo.org>        *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KOPETE_CONTACT_H
#define KOPETE_CONTACT_H

#include <QtCore/QObject>

#include <contact.h>

/**
 * Represents a Kopete contact
 */
class KopeteContact: public QObject, public Contact
{
Q_OBJECT

public:
	KopeteContact(const QString &id)		;

	/**
	 * @return DisplayName property of Kopete contact
	 */
	QString	displayName();

	/**
	 * @return Photo/Avatar of the Kopete contact, or an empty string if not set
	 */
	virtual QString photo();

	/**
	 * @return Kopete online status of the contact.
	 */
	virtual QString onlineStatus();
	
	/**
	 * @return Kopete status message of the contact. 
	 */
	virtual QString onlineStatusMessage();
	
	/**
	 * @return Unread messages in html format
	 */
	virtual QStringList unreadInstantMessages();
	
    /**
     * @return Preferred IM id
     */
	virtual QString instantMessagingId();

Q_SIGNALS:
    void contactChanged(KopeteContact *contact);
	
private Q_SLOTS:
	/** Updates properties using dbus */
	void update(const QString &contactId);

private:
    /** Preferred IM id */
    QString m_instantMessagingId;
    
	/** DisplayName property of Kopete contact */
	QString m_displayName;

	/** Photo/Avatar of the Kopete contact, or an empty string if not set */
	QString m_photo;

	/** Kopete online status of the contact - currently limited to either Online or Offline */
	QString m_onlineStatus;
	
	/** Kopete status message of the contact. */
	QString m_statusMessage;
	
	/** Unread messages in html format */
	QStringList m_unreadMessages;
};

#endif // KOPETE_CONTACT_H
