/*
 *   Copyright (C) 2009 Andrew Coles  <andrew.coles@yahoo.co.uk>
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef COORDINATEBACKEND_H
#define COORDINATEBACKEND_H

#include <KUrl>

enum CoordinateStatus { COORD_UNDEFINED, COORD_2D, COORD_3D };

struct Coordinate {
    CoordinateStatus status;
    double latitude;
    double longitude;
    double altitude;
    double error;
    
    QString countryCode;
    QString countryName;
    QString city;
    
    Coordinate() : status(COORD_UNDEFINED)
    {};

    Coordinate(const double & l1, const double & l2, const double & e,
	       const QString & cc, const QString & cn, const QString & c)
    : status(COORD_2D), latitude(l1), longitude(l2), altitude(-1.0), error(e),
      countryCode(cc), countryName(cn), city(c)
    {};
    
    Coordinate(const double & l1, const double & l2, const double & a, const double & e,
	       const QString & cc, const QString & cn, const QString & c)
    : status(COORD_3D), latitude(l1), longitude(l2), altitude(a), error(e),
      countryCode(cc), countryName(cn), city(c)
    {};
};

class CoordinateBackend {
  
protected:
    CoordinateBackend() {};
    
public:
    virtual ~CoordinateBackend() {};
    virtual Coordinate resolveMAC(const QString &) = 0;
    virtual void addAccessPointData(const QString & mac, const Coordinate &) = 0;
    virtual bool saveAccessPointData() = 0;
    
};

class FileBackend : public CoordinateBackend {

public:
    /**
     * Takes a KUrl pointing to a file containing Wireless Access Point locations.
     */
    FileBackend(const KUrl & toOpen);
  
    virtual Coordinate resolveMAC(const QString &);
    virtual void addAccessPointData(const QString & mac, const Coordinate &);
    virtual bool saveAccessPointData();
    
private:
    class Private;
    Private * const d;
};

#endif
