#!/usr/bin/python
# -*- coding: utf-8 -*-
#
#
#HellaNzb Data Engine: Control and monitor your hellanzb daemon from plasma
#
#Copyright (C) 2009 Mathieu Ducharme <ducharme.mathieu@gmail.com>
#
#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 2 of the License, or
#(at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

import re

from PyQt4.Qt import * 
from PyQt4.QtGui import * 
from PyKDE4.kdecore import *
from PyKDE4 import plasmascript

from xmlrpclib import ServerProxy

#import hellanzbsource

class HellaNZBEngine(plasmascript.DataEngine):

	def __init__(self, parent,args=None):
		plasmascript.DataEngine.__init__(self,parent)


	def init(self):
		self.setMinimumPollingInterval(500)
		self.proxies = {}


	def sourceRequestEvent(self,name):
		return self.updateSourceEvent(name)


	def updateSourceEvent(self, name):
		source = str(name)
		if re.match(r'^http[s]?:\/\/hellanzb:[\w]*\@[\w\.]*:[\d]*\/$', source) == None:
			self.setError(source, 101, "Invalid URI. Must be like: http://hellanzb:changeme@localhost:8760/")
			return False
		
		# Get instance of the Server proxy
		#if (source in self.proxies) == False:
		try:
			self.proxies[source] = ServerProxy(source)
		except:
			self.setError(source, 102, "Can not connect to HellaNZB XMLRPC Server")
			return False
		
		# Get the status from the hellanzb daemon
		try:
			status = self.proxies[source].status()
		except:
			self.setError(source, 103, "Can not retrieve status from HellaNZB XMLRPC Server")
			return False
		
		# Transpose the data from status to data-engine friendly values
		for k, v in status.iteritems():
			if k == 'time':
				v = str(v)
			self.setData(source, k, QVariant(v))
		
		self.setError(source, 0, '')
		return True


	def serviceForSource(self, source):
		return Plasma.DataEngineScript.serviceForSource(self.data_engine_script,source)


	def setError(self, source, code, msg):
		if code > 0:
		    print "ERROR [" + str(code) + "] - " + str(msg)
		self.setData(source, "_error_code", QVariant(code))
		self.setData(source, "_error_message", QVariant(msg))


def CreateDataEngine(parent):
	return HellaNZBEngine(parent)
