/*
 * Copyright (C) 2008 Laura Dragan <aprilush@gmail.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "nepomuktags_engine.h"
#include <Nepomuk/Tag>
#include <Nepomuk/ResourceManager>
#include <Soprano/Model>

NepomukTagsEngine::NepomukTagsEngine(QObject* parent, const QVariantList& args) :
        Plasma::DataEngine(parent, args)
{
    Q_UNUSED(args)
    // setMinimumPollingInterval(60*1000);
    Soprano::Model *mainModel = Nepomuk::ResourceManager::instance()->mainModel();
    connect(mainModel, SIGNAL(statementsAdded()), this, SLOT(repositoryChanged()));
    connect(mainModel, SIGNAL(statementsRemoved()), this, SLOT(repositoryChanged()));
}

NepomukTagsEngine::~NepomukTagsEngine()
{
}

void NepomukTagsEngine::init()
{
    m_min = 65000;
    m_max = 0;
}

bool NepomukTagsEngine::sourceRequestEvent(const QString &name)
{
    updateSourceEvent(name);
    return true;
}

bool NepomukTagsEngine::updateSourceEvent(const QString &name)
{
    QList<Nepomuk::Tag> tagList = Nepomuk::Tag::allTags();
    foreach(Nepomuk::Tag tag, tagList) {
        int count = tag.tagOf().size();
        if (count < m_min) {
            m_min = count;
        }
        if (count > m_max) {
            m_max = count;
        }
        m_tags.insert(tag.label(), QVariant(count));
    }
    setData(name, "tags", m_tags);
    setData(name, "min", m_min);
    setData(name, "max", m_max);
    return true;
}

void NepomukTagsEngine::repositoryChanged()
{
    QList<Nepomuk::Tag> tagList = Nepomuk::Tag::allTags();
    m_tags.clear();foreach(Nepomuk::Tag tag, tagList) {
        int count = tag.tagOf().size();
        if (count < m_min) {
            m_min = count;
        }
        if (count > m_max) {
            m_max = count;
        }
        m_tags.insert(tag.label(), QVariant(count));
    }
    QStringList ss = sources();
    foreach(QString s, ss) {
        setData(s, "tags", m_tags);
        setData(s, "min", m_min);
        setData(s, "max", m_max);
    }
}

K_EXPORT_PLASMA_DATAENGINE(nepomuktags, NepomukTagsEngine)

#include "nepomuktags_engine.moc"
