/*
 * Copyright (C) 2009 Sacha Schutz ( istdasklar@free.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#include "translator_engine.h"

TranslatorEngine::TranslatorEngine(QObject* parent, const QVariantList& args) :
        Plasma::DataEngine(parent, args)
{
    Q_UNUSED(args)
    m_from = "fr";
    m_to   = "en";
    
}

TranslatorEngine::~TranslatorEngine()
{
}

void TranslatorEngine::init()
{
    m_from = "fr";
    m_to   = "en";
    GoogleTranslator * ggtranslator = new GoogleTranslator(this);
    SimpleCryptTranslator * cryptTranslator = new SimpleCryptTranslator(this);


    addTranslator(ggtranslator);
    addTranslator(cryptTranslator);

    m_currentTranslator = ggtranslator->name();
}


bool TranslatorEngine::sourceRequestEvent(const QString &name)
{
    updateSourceEvent(name);
    return true;
}

bool TranslatorEngine::updateSourceEvent(const QString &name)
{
    m_translatorMap[m_currentTranslator]->setTranslate(name,m_from,m_to);
    connect(m_translatorMap[m_currentTranslator], SIGNAL(translated(QString)), this, SLOT (done(QString)));
    return true;
}

void TranslatorEngine::done(QString result)
{
     setData(m_translatorMap[m_currentTranslator]->name(),"result", result);
     setData(m_translatorMap[m_currentTranslator]->name(),"language from", m_from);
     setData(m_translatorMap[m_currentTranslator]->name(),"language to", m_to);


}


K_EXPORT_PLASMA_DATAENGINE(translator, TranslatorEngine)

#include "translator_engine.moc"
