/*
 * Copyright (C) 2009 Sacha Schutz ( istdasklar@free.fr)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; if not, write to the
 * Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

#ifndef TRANSLATOR_ENGINE_H_
#define TRANSLATOR_ENGINE_H_

#include <Plasma/DataEngine>
#include <QMap>

#include "google_translator.h"
#include "simple_crypt_translator.h"


class TranslatorEngine: public Plasma::DataEngine
{
    Q_OBJECT

public:
    TranslatorEngine(QObject* parent, const QVariantList& args);
    ~TranslatorEngine();
    void setLanguageFrom(QString from="fr"){m_from = from;}
    void setLanguageTo(QString to="en"){m_to = to;}
    void setCurrentTranslator(QString name="Google"){m_currentTranslator=name;}
    void addTranslator(AbstractTranslator * translator){m_translatorMap[translator->name()] = translator;}
protected:
    void init();
    bool sourceRequestEvent(const QString& name);

protected slots:
    bool updateSourceEvent(const QString& source);
    void done(QString result);
private:
    QMap<QString, AbstractTranslator*> m_translatorMap; // list of all translator
    QString m_from;  //language code source : example : "fr"
    QString m_to;   // language code result : example : "en"
    QString m_currentTranslator;



};

#endif /* TRANSLATOR_ENGINE_H_ */
