/***************************************************************************
 *   Copyright (C) 2010 Amos Kariuki <amoskahiga@gmail.com>                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA          *
 ***************************************************************************/

/**
 * This weather data engine feteches a wetaher-feed from Yahoo based on the selected location.
 * It also allows a user to search for known weather source locations from yahoo.
 * It is intended to be used as a weather dataengine 'ion', providing data to one of the weather plasmoids.
 */

// Include the "XML for <SCRIPT>" DOM parser
plasmoid.include('xmldom.js');

// Define a class to hold the parsed weather data-feed contents
function WeatherData() {

    this.ttl = "";

    this.location = {
        country     : "",
        region      : "",
        latitude    : "",
        longitude   : ""
    };

    this.units = {
        temperature : "",
        distance    : "",
        pressure    : "",
        speed       : ""
    };

    this.wind = {
        chill       : "",
        direction   : "",
        speed       : ""
    };

    this.atmosphere = {
        humidity    : "",
        visibility  : "",
        pressure    : "",
        rising      : ""
    };

    this.astronomy = {
        sunrise : "",
        sunset  : ""
    };

    this.condition = {
        text        : "",
        code        : "",
        temp        : "",
        dateTime    : ""
    };

    this.forecasts = new Array();   // To store list of available forecasts
}

// Define a class to hold forecast data
function Forecast() {

    this.date   =   "";
    this.low    =   "";
    this.high   =   "";
    this.text   =   "";
    this.code   =   "";
}

// Define a class to hold location data
function Location() {
    this.country    = "";
    this.region     = "";
    this.name       = "";
    this.woeid      = "";
};

/**
 * Parses the XML feed into a WeatherData object
 *
 * @param   xmlData stream containing recieved weather data
 * @return  weatherData object to to hold contents of the parsed weather data
 */
function parseObservationData(xmlData)
{
    var objDom = new XMLDoc(xmlData, xmlError);

    //get the root node
    var objDomTree = objDom.docNode;

    var channelNode = objDomTree.getElements("channel")[0];

    var weatherData = new WeatherData();

    weatherData.ttl = channelNode.getElements("ttl")[0].getText();

    var locationNode = channelNode.getElements("yweather:location")[0];
    weatherData.location.country = locationNode.getAttribute("country");
    weatherData.location.region = locationNode.getAttribute("region");
    weatherData.location.city = locationNode.getAttribute("city");

    var unitsNode = channelNode.getElements("yweather:units")[0];
    weatherData.units.temperature = unitsNode.getAttribute("temperature");
    weatherData.units.distance = unitsNode.getAttribute("distance");
    weatherData.units.pressure = unitsNode.getAttribute("pressure");
    weatherData.units.speed = unitsNode.getAttribute("speed");

    var windNode = channelNode.getElements("yweather:wind")[0];
    weatherData.wind.chill = windNode.getAttribute("chill");
    weatherData.wind.direction = windNode.getAttribute("direction");
    weatherData.wind.speed = windNode.getAttribute("speed");

    var atmosphereNode = channelNode.getElements("yweather:atmosphere")[0];
    weatherData.atmosphere.humidity = atmosphereNode.getAttribute("humidity");
    weatherData.atmosphere.visibility = atmosphereNode.getAttribute("visibility");
    weatherData.atmosphere.pressure = atmosphereNode.getAttribute("pressure");
    weatherData.atmosphere.rising = atmosphereNode.getAttribute("rising");

    var astronomyNode = channelNode.getElements("yweather:astronomy")[0];
    weatherData.astronomy.sunrise = astronomyNode.getAttribute("sunrise");
    weatherData.astronomy.sunset = astronomyNode.getAttribute("sunset");

    var itemNode = channelNode.getElements("item")[0];
    weatherData.location.lat = itemNode.getElements("geo:lat")[0].getText();
    weatherData.location.long = itemNode.getElements("geo:long")[0].getText();

    var conditionNode = itemNode.getElements("yweather:condition")[0];
    weatherData.condition.text = conditionNode.getAttribute("text");
    weatherData.condition.code = conditionNode.getAttribute("code");
    weatherData.condition.temp = conditionNode.getAttribute("temp");
    weatherData.condition.dateTime = conditionNode.getAttribute("date");

    var forecastNodes = itemNode.getElements("yweather:forecast");
    for (var i = 0; i < forecastNodes.length; ++i) {

        var forecast = new Forecast();

        forecast.date = forecastNodes[i].getAttribute("date");
        forecast.low = forecastNodes[i].getAttribute("low");
        forecast.high =  forecastNodes[i].getAttribute("high");
        forecast.text =  forecastNodes[i].getAttribute("text");
        forecast.code = forecastNodes[i].getAttribute("code");

        weatherData.forecasts[i] = forecast;
    }

    return weatherData;

    function xmlError(e) {
        //there was an error, show the user
        print(e);
    }
}

// Print parsed contents (debugging)
function printWeatherData(weatherData) {

    print("weatherData.ttl: " + weatherData.ttl);
    print("weatherData.location.lat: " + weatherData.location.lat);
    print("weatherData.location.long: " + weatherData.location.long);
    print("weatherData.location.country: " + weatherData.location.country);
    print("weatherData.location.region: " + weatherData.location.region);
    print("weatherData.location.city: " + weatherData.location.city);
    print("weatherData.units.temperature: " + weatherData.units.temperature);
    print("weatherData.units.distance: " + weatherData.units.distance);
    print("weatherData.units.pressure: " + weatherData.units.pressure);
    print("weatherData.units.speed: " + weatherData.units.speed);
    print("weatherData.wind.chill: " + weatherData.wind.chill);
    print("weatherData.wind.direction: " + weatherData.wind.direction);
    print("weatherData.wind.speed: " + weatherData.wind.speed);
    print("weatherData.atmosphere.humidity: " + weatherData.atmosphere.humidity);
    print("weatherData.atmosphere.visibility: " + weatherData.atmosphere.visibility);
    print("weatherData.atmosphere.pressure: " + weatherData.atmosphere.pressure);
    print("weatherData.atmosphere.rising: " + weatherData.atmosphere.rising);
    print("weatherData.astronomy.sunrise: " + weatherData.astronomy.sunrise);
    print("weatherData.astronomy.sunset: " + weatherData.astronomy.sunset);
    print("weatherData.condition.text: " + weatherData.condition.text);
    print("weatherData.condition.code: " + weatherData.condition.code);
    print("weatherData.condition.temp: " + weatherData.condition.temp);
    print("weatherData.condition.dateTime: " + weatherData.condition.dateTime);

    for (var i = 0; i < weatherData.forecasts.length; ++i) {

        print("");
        print("forecast.date: " + weatherData.forecasts[i].date);
        print("forecast.low: " + weatherData.forecasts[i].low);
        print("forecast.high: " + weatherData.forecasts[i].high);
        print("forecast.text: " + weatherData.forecasts[i].text);
        print("forecast.code: " + weatherData.forecasts[i].code);
    }
}

/**
 * Fetches the weather data for the specified location for the Yahoo weather feed
 *
 * @param   source containing the woeid ID as well as the requesting source string
 */
function getObservationData(source) {

    // Verify HTTP requests are allowed in case the metadata.desktop was rejected
    if (plasmoid.getUrl) {

        // Get a fixed weather-feed for testing
        var observationJob = plasmoid.getUrl("http://weather.yahooapis.com/forecastrss?w=35216");

        var observationData = "";   // String to hold the raw http/rss/xml weather feed content

        function receive(job, data)
        {
            if (job == observationJob) {
                // Store the data feed contents
                if (data.length) {
                    observationData += data.valueOf();
                }
            }
        }

        function finished(job)
        {
            if (job == observationJob) {

                // We have finished obtaining the data feed. Proceed to parse it.
                var weatherData = parseObservationData(observationData);

                printWeatherData(weatherData);  // Debug output
            } else {
                print("some other job is finished?")
            }
        }

        observationJob.data.connect(receive)
        observationJob.finished.connect(finished)
    } else {
        print("HTTP access denied!")
    }
}

/**
 * Parses the XML feed into a location object
 *
 * @param   xmlData stream containing recieved location data
 * @return  weatherData object to to hold contents of the parsed weather data
 */
function parseLocationData(xmlData) {

    var objDom = new XMLDoc(xmlData, xmlError);

    //get the root node
    var objDomTree = objDom.docNode;

    var resultsNode = objDomTree.getElements("results")[0];

    var placeNodes = resultsNode.getElements("place");

    var locations = new Array();

    for (var i = 0; i < placeNodes.length; ++i) {

        var location = new Location();

        location.country = placeNodes[i].getElements("country")[0].getText();
        location.name = placeNodes[i].getElements("name")[0].getText();
        location.woeid = placeNodes[i].getElements("woeid")[0].getText();

        // TODO: There may be more than one admin location--parse for the most relevant
        //var admin1 = placeNodes[i].getElements("admin1")[0].getAttribute("type");
        location.regios = placeNodes[i].getElements("admin1")[0].getText();

        locations[i] = location;
    }

    return locations;

    function xmlError(e) {
        //there was an error, show the user
        print(e);
    }
}

// Print parsed contents (debugging)
function printLocations(locations) {

    for (var i = 0; i < locations.length; ++i) {
        print("");
        print("Name: " + locations[i].name);
        print("Country: " + locations[i].country);
        print("Region: " + locations[i].region);
        print("Woeid: " + locations[i].woeid);
    }
}

/**
 * Search for a woeid given a certain locations name.
 *
 * @param   location for the desired area (state/city/region/province)
 */
function getLocationData(location) {

    // Verify HTTP requests are allowed in case the metadata.desktop was rejected
    if (plasmoid.getUrl) {

        // Get a fixed weather-feed for testing
        var findLocationJob = plasmoid.getUrl("http://query.yahooapis.com/v1/public/yql?q=select name, woeid, country, admin1, admin2, admin3 from geo.places where text=\"" + location + "\"&format=xml");

        var locationData = "";   // String to hold the raw http/rss/xml location feed content

        function receive(job, data)
        {
            if (job == findLocationJob) {
                // Store the data feed contents
                if (data.length) {
                    locationData += data.valueOf();
                }
            }
        }

        function finished(job)
        {
            if (job == findLocationJob) {

                // We have finished obtaining the data feed. Proceed to parse it.
                var locations = parseLocationData(locationData);
                printLocations(locations);

                //printWeatherData(weatherData);  // Debug output
            } else {
                print("some other job is finished?")
            }
        }

        findLocationJob.data.connect(receive)
        findLocationJob.finished.connect(finished)
    } else {
        print("HTTP access denied!")
    }
}

getLocationData("Jakarta");
//getObservationData();

