/*
 *   Copyright 2008 Aike J Sommer <dev@aikesommer.name>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#ifndef SCREENWIDGET_H
#define SCREENWIDGET_H

#include <QGraphicsItem>
#include <QGraphicsWidget>

#include "resizehandle.h"


class ScreenWidget : public QObject, public QGraphicsItem {
    Q_OBJECT
    public:
        ScreenWidget(QGraphicsItem * parent);
        
        QRectF boundingRect() const;
        void paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget);
        
        void setScreenGeom(QRect geom);
        QRect screenGeom();
        void setScale(qreal scale);
        void setAvailableSizes(QList<QSize> sizes);
        void setAvailablePositions(QList<QPoint> positions);
        void setId(const QString & id);
        
    Q_SIGNALS:
        void resizeRequested(QSize size);
        void moveRequested(QPoint position);
        
    protected Q_SLOTS:
        void resizeHandleDragged(QPointF offset);
        void resizeHandleReleased();
        void moveHandleDragged(QPointF offset);
        void moveHandleReleased();
        
    protected:
        //void mousePressEvent(QGraphicsSceneMouseEvent * event);
        //void mouseMoveEvent(QGraphicsSceneMouseEvent * event);
        //void mouseReleaseEvent(QGraphicsSceneMouseEvent * event);
        void hoverEnterEvent(QGraphicsSceneHoverEvent * event);
        void hoverLeaveEvent(QGraphicsSceneHoverEvent * event);
        
    private:
        QSize draggedToSize();
        QPoint draggedToPosition();
        
        QString m_id;
        
        QRect m_screenGeom;
        QList<QSize> m_sizes;
        QList<QPoint> m_positions;
        qreal m_scale;
        QRectF m_drawnRect;
        
        bool m_resizing;
        QPointF m_resized;
        bool m_moving;
        QPointF m_moved;
        bool m_hovered;
        
        ResizeHandle * m_resizeHandle;
        ResizeHandle * m_moveHandle;
};


class ScreensWidget : public QGraphicsWidget {
    Q_OBJECT
    public:
        ScreensWidget(QGraphicsItem * parent);
};


#endif // SCREENWIDGET_H

