/*
 *   Copyright 2008 Aike J Sommer <dev@aikesommer.name>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2,
 *   or (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */


#ifndef SCREENSENGINE_H
#define SCREENSENGINE_H

#include <Plasma/DataEngine>

#include <kephal/kephal.h>


namespace Kephal {
    class Output;
    class Screen;
}


class ScreensEngine : public Plasma::DataEngine {
    Q_OBJECT
    
    public:
        ScreensEngine(QObject* parent, const QVariantList & args);
        Plasma::Service * serviceForSource(const QString & source);


    protected:
        bool sourceRequestEvent(const QString & name);
        bool updateSourceEvent(const QString & source);
        void init();
        
    private Q_SLOTS:
        void screenResized(Kephal::Screen * s, QSize oldSize, QSize newSize);
        void screenMoved(Kephal::Screen * s, QPoint oldPosition, QPoint newPosition);
        void screenAdded(Kephal::Screen * s);
        void screenRemoved(int id);
        
        void outputConnected(Kephal::Output * o);
        void outputDisconnected(Kephal::Output * o);
        void outputActivated(Kephal::Output * o);
        void outputDeactivated(Kephal::Output * o);
        void outputResized(Kephal::Output * o, QSize oldSize, QSize newSize);
        void outputMoved(Kephal::Output * o, QPoint oldPosition, QPoint newPosition);
        void outputRateChanged(Kephal::Output * o, float oldRate, float newRate);
        void outputRotated(Kephal::Output * o, Kephal::Rotation oldRotation, Kephal::Rotation newRotation);
        void outputReflected(Kephal::Output * o, bool oldX, bool oldY, bool newX, bool newY);
        
        void confirmTimeout(int seconds);
        void confirmed();
        void reverted();
        
    private:
        Kephal::Output * outputForSource(const QString & source);
        void dataForOutput(Kephal::Output * output);
        
        QMap<QString, Plasma::Service *> m_services;
};


#endif // SCREENSENGINE_H
