/*
 * Copyright 2009 Petri Damstén <damu@iki.fi>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <KApplication>
#include <KAboutData>
#include <KCmdLineArgs>
#include "view.h"

int main(int argc, char **argv)
{
    KAboutData about("plasma-window", "plasma-window",
                    ki18n("Plasma Window"), "0.1",
                    ki18n("Run plasma in a window."),
                    KAboutData::License_GPL,
                    ki18n("2009 Petri Damstén"));
    about.addAuthor(ki18n("Petri Damstén"), ki18n("Author"), "damu@iki.fi");
    about.setProgramIconName("plasma");

    KCmdLineOptions options;
    options.add("m");
    options.add("monitor", ki18n("Run window in monitoring (non-interactive) mode."));
    options.add("s");
    options.add("viewsize <size>", ki18n("content size eg. 800x600."));
    options.add("a");
    options.add("appname <name>",
                ki18n("Use alternative application name for configuration file."));

    KCmdLineArgs::init(argc, argv, &about);
    KCmdLineArgs::addCmdLineOptions(options);
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    if (args->isSet("appname")) {
        about.setAppName(args->getOption("appname").toLocal8Bit());
    }

    KApplication app;
    // Background dialog
    KGlobal::locale()->insertCatalog("plasma-desktop");
    // Screen preview widget
    KGlobal::locale()->insertCatalog("libkworkspace");
    // other files in common
    KGlobal::locale()->insertCatalog("plasma-shells-common");
    WindowView view(!args->isSet("monitor"));
    if (args->isSet("viewsize")) {
        QStringList list = args->getOption("viewsize").split('x');
        if (list.count() == 2) {
            int w = list[0].toInt();
            int h = list[1].toInt();
            if (w > 0 && h > 0) {
                view.resize(w, h);
            }
        }
    }
    view.show();
    return app.exec();
}
