/*
 *   Copyright 2009 by Shafqat Bhuiyan <priomsrb@gmail.com>
 *   Copyright 2009 by Egor Tsinko <etsinko@gmail.com>
 *
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU Library General Public License as
 *   published by the Free Software Foundation; either version 2, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this program; if not, write to the
 *   Free Software Foundation, Inc.,
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#ifndef SCRIPTED_IMAGE_WALLPAPER_H
#define SCRIPTED_IMAGE_WALLPAPER_H

#include <Plasma/Wallpaper>
#include <KProcess>
#include <QTimer>
#include "ui_config.h"

class ScriptedImage : public Plasma::Wallpaper
{
    Q_OBJECT
    public:
        ScriptedImage(QObject* parent, const QVariantList& args);
        ~ScriptedImage();

        virtual void save(KConfigGroup &config);
        virtual void paint(QPainter* painter, const QRectF& exposedRect);
        virtual QWidget* createConfigurationInterface(QWidget* parent);

    Q_SIGNALS:
        void settingsChanged(bool modified);

    protected:
        virtual void init(const KConfigGroup &config);        

    protected slots:
        void settingsModified();

    private slots:
        void setWallpaperFromScriptOutput();
        void setWallpaper(const QImage &image);
        void setWallpaperToFallbackImage();
        void runScript();

    private:
        QString scriptPath;
        KProcess scriptProcess;
        float secondsToRunScript;
        QTimer scriptTimer;
        QString wallpaperPath;
        QPixmap wallpaperPixmap;
        QString fallbackImagePath;
        QPixmap fallbackImagePixmap;
        Plasma::Wallpaper::ResizeMethod resizeMethod;
        QSizeF previousSize;

        Ui::Config ui;

        void setWallpaperPathToScriptOutput();
        void redrawWallpaper();
        void setupScript();
        void loadFallbackImage();
        void setFallbackImageToBlack();
};

K_EXPORT_PLASMA_WALLPAPER(scripted_image, ScriptedImage)

#endif
