#include "koratbutton.h"

#include "koratdefines.h"

#include <plasma/widgets/toolbutton.h>
#include <plasma/widgets/iconwidget.h>
#include <plasma/framesvg.h>
#include <plasma/dialog.h>
#include <plasma/theme.h>

#include <KDebug>

#include <QGraphicsLinearLayout>
#include <QGraphicsWidget>
#include <QToolButton>
#include <QAction>

namespace Korat
{

class ButtonPrivate
{
public:
    ButtonPrivate(Button *button)
        : q(button)
        , checked(false)
        , checkedBackground(new Plasma::FrameSvg())
        , styleMode(Button::StylePlain)
        , graphicsWidget(0)
        , iconWidget(new Plasma::IconWidget())
        , testIconWidget(new Plasma::IconWidget())
        , layout(new QGraphicsLinearLayout())
    {
        testIconWidget->setDrawBackground(true);
    }

    void clicked()
    {
        buttonAction->toggle();
    }

    void calculateWidths()
    {
        qreal width = 0;

        widths.clear();

        testIconWidget->setOrientation(Qt::Horizontal);
        testIconWidget->setIcon(buttonAction->icon());
        testIconWidget->setText(QString());

        width = testIconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE).width();
        widths << width;

        testIconWidget->setOrientation(Qt::Horizontal);
        testIconWidget->setText(buttonAction->text());

        width = testIconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE).width();
        widths << width;

        testIconWidget->setOrientation(Qt::Vertical);
        width = testIconWidget->sizeFromIconSize(LARGEST_ICON_SIZE).width();
        widths << width;
    }

    Button *q;

    bool checked;
    Plasma::FrameSvg *checkedBackground;

    Button::StyleMode styleMode;
    QList<qreal> widths;

    QAction *buttonAction;
    QGraphicsWidget *graphicsWidget;

    Plasma::IconWidget *iconWidget;
    Plasma::IconWidget *testIconWidget;

    QGraphicsLinearLayout *layout;
};

Button::Button(QAction *action, StyleMode styleMode, QGraphicsItem *parent)
    : Item(parent)
    , d(new ButtonPrivate(this))
{
    //setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    setContentsMargins(0,0,0,0);

    d->checkedBackground->setImagePath("widgets/frame");
    d->checkedBackground->setElementPrefix("sunken");

    d->buttonAction = action;
    d->styleMode = styleMode;

    d->iconWidget->setDrawBackground(true);
    d->iconWidget->setText(d->buttonAction->text());
    d->iconWidget->setIcon(d->buttonAction->icon());
    if (d->buttonAction->isCheckable()) {
        connect(d->iconWidget,SIGNAL(clicked()),this,SLOT(clicked()));
    }

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->layout->addItem(d->iconWidget);

    setLayout(d->layout);
}

Button::Button(QGraphicsWidget *widget, QAction *action, StyleMode styleMode, QGraphicsItem *parent)
    : Item(parent)
    , d(new ButtonPrivate(this))
{
    setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);
    setContentsMargins(0,0,0,0);

    d->graphicsWidget = widget;
    d->buttonAction = action;
    d->styleMode = styleMode;

    d->iconWidget->setDrawBackground(true);
    d->iconWidget->setText(d->buttonAction->text());
    d->iconWidget->setIcon(d->buttonAction->icon());

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->layout->addItem(d->iconWidget);

    setLayout(d->layout);
}

Button::StyleMode Button::styleMode(Button::StyleMode styleMode) const
{
    return d->styleMode;
}

void Button::setStyleMode(Button::StyleMode styleMode)
{
    d->styleMode = styleMode;
}

QList<WeightList> Button::availableWidths()
{
    QList<WeightList> result;
    WeightList weightList;
    weightList << (int)BoxButtonLarge << (int)BoxButtonSmallWithText
        << (int)BoxButtonSmall;
    result << weightList;

    return result;
}

qreal Button::getWidth(const WeightList &weightList)
{
    d->calculateWidths();
    qreal width = 0;
    switch (weightList.at(0)) {
    case BoxButtonSmall:
        width = d->widths.at(0);
        break;
    case BoxButtonSmallWithText:
        width = d->widths.at(1);
        break;
    case BoxButtonLarge:
        width = d->widths.at(2);
        break;
    default:
        kWarning() << "Trouble:" << weightList.at(0);
    }

//X     qreal left,top,right,bottom;
//X     d->background->getMargins(left,top,right,bottom);
//X 
//X     return width + left + right;
    return width;
}

void Button::adjustWidth(const WeightList &weightList)
{
    switch (weightList.at(0)) {
    case BoxButtonSmall:
        d->iconWidget->setText(QString());
        d->iconWidget->setOrientation(Qt::Vertical);
        d->iconWidget->setMinimumSize(d->iconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE));
        d->iconWidget->setMaximumSize(d->iconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE));
        break;
    case BoxButtonSmallWithText:
        d->iconWidget->setText(d->buttonAction->text());
        d->iconWidget->setOrientation(Qt::Horizontal);
        d->iconWidget->setMinimumSize(d->iconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE));
        d->iconWidget->setMaximumSize(d->iconWidget->sizeFromIconSize(SMALLEST_ICON_SIZE));
        break;
    case BoxButtonLarge:
        d->iconWidget->setText(d->buttonAction->text());
        d->iconWidget->setOrientation(Qt::Vertical);
        d->iconWidget->setMinimumSize(d->iconWidget->sizeFromIconSize(LARGEST_ICON_SIZE));
        d->iconWidget->setMaximumSize(d->iconWidget->sizeFromIconSize(LARGEST_ICON_SIZE));
        break;
    default:
        kWarning() << "Trouble:" << weightList.at(0);
    }
}

void Button::paint(QPainter *painter,const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    Q_UNUSED(option);
    Q_UNUSED(widget);
    if (d->buttonAction->isCheckable()) {
        d->checkedBackground->resizeFrame(size());
        if (d->buttonAction->isChecked()) {
            d->checkedBackground->paintFrame(painter);
        }
    }
}

} // namespace Korat

#include "koratbutton.moc"
// vim: sw=4 sts=4 et tw=100
