/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#include "koratmainwindow.h"
#include "koratmainwindowgraphics.h"

#include <plasma/framesvg.h>
#include <plasma/corona.h>

#include <QGraphicsLinearLayout>
#include <QGraphicsScene>
#include <QGraphicsView>
#include <QHBoxLayout>

namespace Korat
{
class MainWindowPrivate
{
public:
    MainWindowPrivate(MainWindow *mainwindow)
        : q(mainwindow)
        , scene(new Plasma::Corona())
        , view(new QGraphicsView())
        , graphicsWidget(new MainWindowGraphics())
        , layout(new QHBoxLayout())
    {
    }

    MainWindow *q;

    Plasma::Corona *scene;
    QGraphicsView *view;
    MainWindowGraphics *graphicsWidget;

    QHBoxLayout *layout;
};

MainWindow::MainWindow(QWidget *parent)
    : QWidget(parent)
    , d(new MainWindowPrivate(this))
{
    setAttribute(Qt::WA_TranslucentBackground);

    QPalette pal = palette();
    pal.setColor(backgroundRole(), Qt::transparent);
    setPalette(pal);

    d->scene->addItem(d->graphicsWidget);

    d->view->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    d->view->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    d->view->setFrameShape(QFrame::NoFrame);
    d->view->viewport()->setAutoFillBackground(false);
    d->view->setContentsMargins(0,0,0,0);
    d->view->setScene(d->scene);

    d->layout->setSpacing(0);
    d->layout->setContentsMargins(0,0,0,0);
    d->layout->addWidget(d->view);

    setLayout(d->layout);

    //setSizePolicy(QSizePolicy::Fixed,QSizePolicy::Fixed);

    sizeUpdate();
}

MainWindow::~MainWindow()
{

}

ToolBar *MainWindow::toolBar()
{
    return d->graphicsWidget->toolBar();
}

void MainWindow::setToolBar(ToolBar *bar)
{
    d->graphicsWidget->setToolBar(bar);
}

Drawer *MainWindow::drawer(Corner corner)
{
    return d->graphicsWidget->drawer(corner);
}

void MainWindow::addDrawer(Drawer *widget, Corner corner)
{
    d->graphicsWidget->addDrawer(widget,corner);
}

Dock *MainWindow::dock(Edge edge)
{
    return d->graphicsWidget->dock(edge);
}

void MainWindow::setDock(Dock *widget, Edge edge)
{
    d->graphicsWidget->setDock(widget,edge);
}

QGraphicsWidget *MainWindow::centralWidget()
{
    return d->graphicsWidget->centralWidget();
}

void MainWindow::setCentralWidget(QGraphicsWidget *widget)
{
    d->graphicsWidget->setCentralWidget(widget);
}
void MainWindow::resizeEvent(QResizeEvent *e)
{
//    QWidget::resizeEvent(e);

//    d->view->resize(contentsRect().size());

    if (d->graphicsWidget) {
        d->graphicsWidget->resize(e->size());
        resize(d->graphicsWidget->size().toSize());
        //d->view->resize(d->graphicsWidget->size().toSize());
        d->view->centerOn(d->graphicsWidget);
    }
}

void MainWindow::paintEvent(QPaintEvent *e)
{
    QPainter p(this);
    p.setClipRect(e->rect());
    p.setCompositionMode(QPainter::CompositionMode_Source);
    p.fillRect(e->rect(),Qt::transparent);
}

bool MainWindow::event(QEvent *e)
{
    if (e->type() == QEvent::Paint) {
        QPainter p(this);
        p.setCompositionMode(QPainter::CompositionMode_Source);
        p.fillRect(rect(), Qt::transparent);
    }
    
    return QWidget::event(e);
}

void MainWindow::sizeUpdate()
{
//    d->view->centerOn(d->graphicsWidget);
}

} // namespace Korat

#include "koratmainwindow.moc"
// vim: sw=4 sts=4 et tw=100
