/***************************************************************************
 *   Copyright (C) 2009 by Wang Hoi <zealot.hoi@gmail.com>                 *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA .        *
 ***************************************************************************/

#ifndef KORATMAINWINDOWGRAPHICS_H
#define KORATMAINWINDOWGRAPHICS_H

#include "koratbar_export.h"
#include "koratbar.h"
#include "koratdrawer.h"
#include "koratdock.h"
#include "koratstatusbar.h"

#include <QGraphicsWidget>

namespace Korat
{
class MainWindowGraphicsPrivate;
class KORATBAR_EXPORT MainWindowGraphics : public QGraphicsWidget
{
    Q_OBJECT
public:
    explicit MainWindowGraphics(QGraphicsItem *parent=0);
    virtual ~MainWindowGraphics();

    ToolBar *toolBar();
    void setToolBar(ToolBar *bar);

    StatusBar *statusBar();
    void setStatusBar(StatusBar *bar);

    QString title() const;
    void setTitle(const QString &title);

    QIcon icon() const;
    void setIcon(const QIcon &icon);

    virtual Drawer *drawer(Corner corner);
    virtual void addDrawer(Drawer *widget, Corner corner);

    virtual Dock *dock(Edge edge);
    virtual void setDock(Dock *widget, Edge edge);

    virtual QGraphicsWidget *centralWidget();
    virtual void setCentralWidget(QGraphicsWidget *widget);

Q_SIGNALS:
    void sizeChanged();

protected:

    virtual void resizeEvent(QGraphicsSceneResizeEvent * event);

    virtual void paint(QPainter *painter,
                       const QStyleOptionGraphicsItem *option,
                       QWidget *widget = 0);
protected Q_SLOTS:
    void themeUpdated();

private:
    MainWindowGraphicsPrivate *d;
};
} // namespace Korat


#endif // KORATMAINWINDOW_H
