#include "widget.h"

#include "node.h"

#include <QMouseEvent>
#include <QVariant>

#include <qdebug.h>
#include <phonon/audiooutput.h>
#include <phonon/mediaobject.h>

using namespace Quasar;

Widget::Widget(const QString &mediaFile)
    : QGLWidget()
{
    m_media = new Phonon::MediaObject(this);
    m_media->setCurrentSource(mediaFile);
    Phonon::createPath(m_media, new Phonon::AudioOutput(this));
}

Widget::~Widget()
{
    //delete m_input;
}

void Widget::initializeGL()
{
    glClearColor(0.0, 0.0, 0.0, 1);

    m_input = new PhononInputNode();
    connect(m_input, SIGNAL(needUpdate()), SLOT(updateGL()));
    m_compo.addNode(m_input);
    m_displayNode = m_compo.createNode("/output/render");
    m_saturationNode = m_compo.createNode("/color/edgedetect");
    Quasar::link(m_input, "output", m_saturationNode, "texture");
    Quasar::link(m_saturationNode, "output", m_displayNode, "texture");
    Phonon::createPath(m_media, m_input);
    m_compo.prepare();
    connect(m_media, SIGNAL(finished()), m_media, SLOT(play()));
    m_media->play();
}

void Widget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    m_compo.execute();
    glFlush();
}

void Widget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width, 0, height, -999999, 999999);
    glMatrixMode(GL_MODELVIEW);
}

void Widget::keyPressEvent(QKeyEvent *e)
{
    switch (e->key()) {
    case Qt::Key_S:
        if (e->modifiers() & Qt::ShiftModifier) {
            m_saturationNode->setPropertyValue("Threshold", m_saturationNode->propertyValue("Threshold").toDouble() + 0.1);
        } else {
            m_saturationNode->setPropertyValue("Threshold", m_saturationNode->propertyValue("Threshold").toDouble() - 0.1);
        }
        updateGL();
        break;
    }
}

void Widget::mousePressEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_displayNode->setPropertyValue("point", QPointF(pt.x(), height() - pt.y()));
    update();
}

void Widget::mouseMoveEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_displayNode->setPropertyValue("point", QPointF(pt.x(), height() - pt.y()));
    update();
}
