#ifndef QUASAR_NODELOADER_H
#define QUASAR_NODELOADER_H

#include <QtCore/qhash.h>
#include <QtCore/qmap.h>
#include <QtCore/qstringlist.h>
#include <QtCore/qstring.h>

namespace Quasar
{
    class Node;

    class NodeCreator {
    public:
        virtual ~NodeCreator()
        {}
        virtual Node *createNode() = 0;
    };

    class NodeLoader
    {
    public:
        static NodeLoader *self();

    public:
        Node *createNode(const QString &url);

        QStringList availableGroups() const;
        QStringList filtersInGroup(const QString &group) const;

        QStringList availableFilters() const;

    public:
        NodeLoader();
        ~NodeLoader();

    private:
        void loadBuiltins();
        void registerNode(const QString &name,
                          NodeCreator *creator);
    private:
        QHash<QString, NodeCreator*> m_nodes;
        QMap<QString, QStringList> m_groups;
    };
}

#endif
