#ifndef QUASAR_TEXTURE_H
#define QUASAR_TEXTURE_H

#include <GL/gl.h>

#include <QtGui/QImage>
#include <QtCore/QVariant>
#include <QtCore/QSharedDataPointer>
#include <QtCore/QRect>

class QByteArray;

/* thin wrapper around gl texture */
namespace Quasar
{
    class Texture
    {
    public:
        Texture();
        explicit Texture(GLint id);
        Texture(const Texture &other);
        Texture(const QByteArray &data,
                int width, int height,
                GLenum format);
        Texture(const char *data,
                int width, int height,
                GLenum format);
        ~Texture();

        bool isNull() const;

        GLuint id() const;

        int width() const;
        int height() const;
        GLenum format() const;

        void updateRect(const QRect &rect, const QByteArray &data,
                        GLenum f);
        void updateRect(const QRect &rect, const char *data,
                        GLenum f);

        Texture &operator=(const Texture &other);

        QImage toImage() const;

        operator QVariant() const;
    private:
        class Private;
        QSharedDataPointer<Private> d;
        friend class Private;
    };

}

Q_DECLARE_TYPEINFO(Quasar::Texture, Q_MOVABLE_TYPE);
Q_DECLARE_METATYPE(Quasar::Texture)

#endif
