#include "renderoutput_p.h"

#include "texture.h"

#include <QPointF>
#include <QVariant>

Quasar::RenderOutputNode::RenderOutputNode()
    : Node("Render")
{
    Quasar::Property tex = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("texture"),
        QVariant::UserType,
        QVariant(),
        QString::fromLatin1("Loaded texture"),

        QVariant(),
        QVariant(),
        QVariant(),
    };
    addProperty(QString::fromLatin1("texture"), tex);

    Quasar::Property pt = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("point"),
        QVariant::PointF,
        QPointF(0, 0),
        QString::fromLatin1("Anchor point"),

        QPointF(0, 0),
        QVariant(),
        QVariant(),
    };
    addProperty(QString::fromLatin1("point"), pt);
}

bool Quasar::RenderOutputNode::execute()
{
    QVariant res = propertyValue("texture");
    Texture tex = qvariant_cast<Texture>(res);

    QPointF pt = propertyValue("point").toPointF();

    glPushMatrix();
    glLoadIdentity();
    glTranslatef(pt.x(), pt.y(), 0);
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, tex.id());
    glBegin(GL_TRIANGLE_STRIP);
    glTexCoord2f(0, 0); glVertex3f(0, 0, 0);
    glTexCoord2f(0, 1); glVertex3f(0, tex.height(), 0);
    glTexCoord2f(1, 0); glVertex3f(tex.width(), 0, 0);
    glTexCoord2f(1, 1); glVertex3f(tex.width(), tex.height(), 0);
    glEnd();
    glPopMatrix();

    return !tex.isNull();
}
