#define GL_GLEXT_PROTOTYPES
#include "saturationnode_p.h"


#include <QVariant>
#include <QDebug>

#include <GL/glext.h>
#include <GL/gl.h>

using namespace Quasar;

static const char *vertShaderText =
    "void main() {\n"
    "   gl_FrontColor = gl_Color;\n"
    "   gl_TexCoord[0] = gl_MultiTexCoord0;\n"
    "   gl_Position = gl_ModelViewProjectionMatrix * gl_Vertex;\n"
    "}\n";

static const char *fragShaderText =
    "const vec3 lumCoeff = vec3(0.2125, 0.7154, 0.0721);\n"
    "uniform sampler2D srcTex;\n"
    "uniform float Alpha;\n"
    "void main() {\n"
    "    vec3 fragColor = texture2D(srcTex,  gl_TexCoord[0].st).rgb;\n"
    "    vec3 intensity = vec3(dot(fragColor, lumCoeff));\n"
    "    vec3 color = mix(intensity, fragColor, Alpha);\n"
    "    gl_FragColor =  vec4(color, 1.0);\n"
    "}\n";

SaturationNode::SaturationNode()
    : FilterNode("Saturation")
{
    setShaderSources(vertShaderText, fragShaderText);
    Quasar::Property aprop = {
        Quasar::Property::InputProperty,

        QString::fromLatin1("Alpha"),
        QVariant::Double,
        1.0,
        QString::fromLatin1("Saturation alpha"),

        1.0,
        0,
        10,
    };
    addProperty(QString::fromLatin1("Alpha"), aprop);
}

QRectF SaturationNode::domainOfDefinition(const QRectF &rect) const
{
    return rect;
}

QRectF SaturationNode::regionOfInterest(const QRectF &rect) const
{
    return rect;
}

void SaturationNode::aboutToExecute()
{
}

void SaturationNode::programChanged(GLuint prog)
{
    GLuint texLoc = glGetUniformLocationARB(prog, "srcTex");
    glUniform1iARB(texLoc, 0);
    QVariant var = propertyValue("Alpha");
    float res = var.toDouble();
    m_alphaLoc =  glGetUniformLocationARB(prog, "Alpha");
    glUniform1fARB(m_alphaLoc, res);
}
