#include "widget.h"

#include <QMouseEvent>

#include <qdebug.h>

using namespace Quasar;

Widget::Widget(const QString &imgFile)
    : QGLWidget(),
      m_image(imgFile)
{
    m_image = QGLWidget::convertToGLFormat(m_image);
}

void Widget::initializeGL()
{
    glClearColor(0.3, 0.3, 0.3, 1);
    qDebug()<<"Image null = "<<m_image.isNull();
    qDebug()<<"\t w = "<<m_image.width()
            <<", h = "<<m_image.height();
    m_tex = Texture((const char*)m_image.bits(),
                    m_image.width(), m_image.height(), GL_RGBA);
    m_mesh = new Mesh(m_tex, 2);
}

void Widget::paintGL()
{
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
    m_mesh->render(m_point);
    glFlush();
}

void Widget::resizeGL(int width, int height)
{
    glViewport(0, 0, width, height);
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho(0, width, 0, height, -999999, 999999);
    glMatrixMode(GL_MODELVIEW);
}

void Widget::mousePressEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_point = QPointF(pt.x(), height() - pt.y());
    update();
}

void Widget::mouseMoveEvent(QMouseEvent *event)
{
    QPointF pt = event->pos();

    m_point = QPointF(pt.x(), height() - pt.y());
    update();
}
